@@ -60,6 +60,9 @@ extern "C" {
 #  define restrict
 # endif /* !__GNUG__ */
 #endif /* !__cplusplus */
+#ifdef _MSC_VER
+# define restrict
+#endif /* !_MSC_VER */
 
 /* Boolean values */
 /* From Wikipedia:
@@ -262,7 +265,8 @@ extern "C" {
     return nco_mta_dlm;} /* [sng] Multi-argument delimiter */
   void nco_mta_dlm_set(char *nco_mta_dlm_arg){nco_mta_dlm=nco_mta_dlm_arg;} /* [sng] Multi-argument delimiter */
   
-  unsigned short nco_baa_cnv=0; /* [enm] Bit-Adjustment Algorithm */
+  //unsigned short nco_baa_cnv=0; /* [enm] Bit-Adjustment Algorithm defaults to Bit Grooming (until 20200715) */
+  unsigned short nco_baa_cnv=5; /* [enm] Bit-Adjustment Algorithm defaults to Bit Rounding (after 20200715) */
   unsigned short nco_baa_cnv_get(void){return nco_baa_cnv;} /* [enm] Bit-Adjustment Algorithm */
 
   unsigned short nco_bnr_cnv=0; /* [enm] Binary byte-ordering convention to produce (native or byte-swapped) */
@@ -348,7 +352,7 @@ extern "C" {
 # define NCO_VERSION_MINOR 9
 #endif /* !NCO_VERSION_MINOR */
 #ifndef NCO_VERSION_PATCH
-# define NCO_VERSION_PATCH 3
+# define NCO_VERSION_PATCH 5
 #endif /* !NCO_VERSION_PATCH */
 #ifndef NCO_VERSION_NOTE
 # define NCO_VERSION_NOTE "" /* Blank for final versions, non-blank (e.g., "-beta37") for pre-release versions */
@@ -358,7 +362,7 @@ extern "C" {
 # define NCO_LIB_VERSION ( NCO_VERSION_MAJOR * 100 + NCO_VERSION_MINOR * 10 + NCO_VERSION_PATCH )
 #endif /* !NCO_LIB_VERSION */
 #ifndef NCO_VERSION
-# define NCO_VERSION "4.9.3"
+# define NCO_VERSION "4.9.5"
 #endif /* !NCO_VERSION */
 
 /* Compatibility tokens new to netCDF4 netcdf.h: */
@@ -727,6 +731,8 @@ extern "C" {
     nco_baa_set, /* 2 Bit Set (option since 20160117) */
     nco_baa_dgr, /* 3 Digit Rounding (GCD19 option since 20190121) */
     nco_baa_bg2, /* 4 Bit Groom version 2 dynamic masks (option since 20190203) */
+    nco_baa_rnd, /* 5 Bit rounding */
+    nco_baa_sh2, /* 6 Bit Half-shave  */
   }; /* end nco_baa_cnv */
 
   enum nco_bnr_cnv{ /* [enm] Binary byte-ordering convention to employ (native or byte-swapped) */
@@ -820,7 +826,7 @@ extern "C" {
 
     char *max_sng; /* User-specified string for dimension maximum */
     char *min_sng; /* User-specified string for dimension minimum */
-    char *mro_sng; /* User-specified string for multi-record output */
+    char *ilv_sng; /* User-specified string for interleave stride */
     char *ssc_sng; /* User-specified string for dimension subcycle */
     
     char *rbs_sng; /* Used by ncra, ncrcat to re-base record coordinate (holds unit attribute from first file) */
@@ -834,12 +840,15 @@ extern "C" {
     int lmt_typ; /* crd_val or dmn_idx */
 
     long cnt; /* # of valid elements in this dimension (including effects of stride and wrapping) */
-    long ssc; /* Subcycle of hyperslab */
     long end; /* Index to end of hyperslab */
+    long ilv; /* Interleave stride */
     long max_idx; /* Index of maximum requested value in dimension */
     long min_idx; /* Index of minimum requested value in dimension */
+    long srd; /* Stride of hyperslab */
+    long srt; /* Index to start of hyperslab */
+    long ssc; /* Subcycle of hyperslab */
 
-    /* Following four flags are used only by multi-file operators ncra and ncrcat: */
+    /* Following six counters are used only by multi-file operators ncra and ncrcat: */
     long idx_end_max_abs; /* [idx] Maximum allowed index in record dimension (multi-file record dimension only) */
     long rec_dmn_sz; /* [nbr] Number of records in this file (multi-file record dimension only) */
     long rec_in_cml; /* [nbr] Cumulative number of records in all files opened so far (multi-file record dimension only) */
@@ -847,11 +856,10 @@ extern "C" {
     long rec_skp_ntl_spf; /* [nbr] Records skipped in initial superfluous files (multi-file record dimension only) */
     long rec_skp_vld_prv; /* [nbr] Records skipped since previous good one (multi-file record dimension only) */
 
-    long srd; /* Stride of hyperslab */
-    long srt; /* Index to start of hyperslab */
-
-    nco_bool flg_mro; /* True for multi-record output (used by ncra only) */
+    nco_bool flg_ilv; /* True for interleaved output (used by ncra and ncrcat only) */
     nco_bool flg_input_complete; /* True for multi-file operators when no more files need be opened */
+    nco_bool flg_mro; /* True for multi-record output (used by ncra only) */
+    nco_bool flg_mso; /* True for multi-subcycle output (used by ncra only) */
 
     nco_bool is_rec_dmn; /* True if record dimension, else False */
     nco_bool is_usr_spc_lmt; /* True if any part of limit is user-specified, else False */
@@ -1090,7 +1098,14 @@ extern "C" {
     nco_edg_crt, /* [enm] Edges/arcs are Cartesian/planar lines and neglect curvature */
   } nco_edg_typ_enm;
 
-  typedef enum nco_xtr_typ_enm 
+  typedef enum nco_wgt_typ_enm{
+    nco_wgt_nil=0,
+    nco_wgt_con, /* Conservative algorithm */
+    nco_wgt_dwe, /* Distance-weighted extrapolation */
+    nco_wgt_bln  /* Bilinear interpolation */
+  } nco_wgt_typ_enm;
+
+typedef enum nco_xtr_typ_enm
     { /* [enm] Extrapolation type enum */
      nco_xtr_fll_lnr=0, // Perform linear extrapolation using two nearest valid neighbors
      nco_xtr_fll_ngh, // Set extrapolated value to value of nearest valid neighbor
@@ -1106,7 +1121,7 @@ extern "C" {
     } nco_xtr_typ_enm;
 
   /* CF Coordinates structure (20160503: Used only in ncks.c for to infer grids from CF coordinates convention) */
-  typedef struct{ /* clm_bnd_sct */
+  typedef struct{ /* cr_crd_sct */
     char *crd_nm[2]; /* [sng] Coordinate names */
     char *crd_sng; /* [sng] Coordinates attribute value */
     char *dmn_nm[2]; /* [sng] Dimension names */
@@ -1119,32 +1134,6 @@ extern "C" {
     nco_bool crd; /* [flg] CF coordinates information is complete */
   } cf_crd_sct; /* end CF coordinates structure */
 
-  /* Climatology bounds structure (20160503: Used only in ncra.c for climos) */
-  typedef struct{ /* clm_bnd_sct */
-    char *bnd_dmn_nm; /* [sng] Bounds dimension name name */
-    char *clm_bnd_nm; /* [sng] Climatology bounds variable name (to create) */
-    char *tm_bnd_nm; /* [sng] Time bounds variable name (to delete) */
-    char *tm_crd_nm; /* [sng] Name of time coordinate variable */
-    char *cln_val; /* [sng] Bounds calendar value */
-    char *unt_val; /* [sng] Bounds units value */
-    double val[2]; /* [frc] Climatology bounds variable attribute values */
-    int clm_bnd_id_in; /* [id] Variable ID for clm_bnds in input */
-    int clm_bnd_id_out; /* [id] Variable ID for clm_bnds in output */
-    int tm_bnd_id_in; /* [id] Variable ID for tm_bnds in input */
-    int tm_bnd_id_out; /* [id] Variable ID for tm_bnds in output */
-    int tm_crd_id_in; /* [id] Variable ID for tm_crd in input */
-    int tm_crd_id_out; /* [id] Variable ID for tm_crd in output */
-    int dmn_ids[2]; /* [idx] Dimension IDs for new bounds variable */
-    long dmn_srt_srt[2]; /* [idx] Start indices for retrieving start bounds */
-    long dmn_srt_end[2]; /* [idx] Start indices for retrieving end bounds */
-    nc_type type; /* [enm] Type of (time and) climatology bounds variable(s) */
-    nco_bool bnd2clm; /* [flg] Convert time bounds to climatology bounds */
-    nco_bool clm2bnd; /* [flg] Convert climatology bounds to time bounds */
-    nco_bool clm2clm; /* [flg] Convert climatology bounds to climatology bounds */
-    nco_bool clm_bnd_in; /* [flg] Climatology bounds appear in input */
-    nco_bool tm_bnd_in; /* [flg] Time bounds appear in input */
-  } clm_bnd_sct; /* end climatology bounds structure */
-
   /* Terraref structure */
   typedef struct{ /* trr_sct */
     // File names specifiable with individual command line switches
@@ -1230,8 +1219,10 @@ extern "C" {
     double lat_nrt; /* [dgr] Latitude of northern edge of grid */
     double lon_est; /* [dgr] Longitude of eastern edge of grid */
     double sgs_nrm; /* [frc] Sub-gridscale normalization */
+    double xtr_xpn; /* [frc] Exponent of distance in extrapolation (absolute value) */
     long lat_nbr; /* [nbr] Number of latitudes in destination grid */
     long lon_nbr; /* [nbr] Number of longitudes in destination grid */
+    long xtr_nsp; /* [nbr] Extrapolation number of source points */
     nco_grd_2D_typ_enm grd_typ; /* [enm] Destination grid-type enum */
     nco_grd_lat_drc_enm lat_drc; /* [enm] Latitude grid-direction enum */
     nco_grd_lat_typ_enm lat_typ; /* [enm] Latitude grid-type enum */
@@ -1240,6 +1231,7 @@ extern "C" {
     nco_ply_tri_mth_typ_enm ply_tri_mth; /* [enm] Polygon-to-triangle decomposition method */ 
     nco_edg_typ_enm edg_typ; /* [enm] Arc-type for triangle edges */
     nco_xtr_typ_enm xtr_mth; /* [enm] Extrapolation method */
+    nco_wgt_typ_enm wgt_typ; /* [enm] Weight generation type */
     // Other internal data and metadata 
     char **xtn_var; /* [sng] Extensive variables */
     char *cmd_ln; /* [sng] Command-line */
@@ -1264,12 +1256,13 @@ extern "C" {
     nco_bool flg_grd; /* [flg] Create SCRIP-format grid file */
     nco_bool flg_grd_dst; /* [flg] User-specified destination grid */
     nco_bool flg_grd_src; /* [flg] User-specified input grid */
-    nco_bool flg_wgt; /* [flg] User-specified mapping weights */
     nco_bool flg_msk_out; /* [flg] Add mask to output */
     nco_bool flg_nfr; /* [flg] Infer SCRIP-format grid file */
     nco_bool flg_rnr; /* [flg] Renormalize destination values by valid area */
     nco_bool flg_stg; /* [flg] Write staggered grid with FV output */
+    nco_bool flg_uio; /* [flg] Unbuffered I/O (NC_SHARE) for netCDF3 files */
     nco_bool flg_usr_rqs; /* [flg] User-requested regridding */
+    nco_bool flg_wgt; /* [flg] User-specified mapping weights */
     size_t hdr_pad; /* [B] Pad at end of header section (for map-file only) */
   } rgr_sct; /* end Regrid structure */
 
@@ -1589,7 +1582,7 @@ extern "C" {
     char *nm; /* [sng] Variable name */
     char *nm_fll; /* [sng] Absolute variable name (needed for GTT search for object by full name) */
     char fmt[5]; /* [sng] Hint for printf()-style formatting */
-    double *wgt_sum; /* [frc] Running sum of per-file weights (ncra/ncea only) */
+    double *wgt_sum; /* [frc] Running sum (over records) of per-record or per-file weights (ncra/ncea only) */
     double wgt_crr; /* [frc] Weight of current record (ncra/ncea only) */
     dmn_sct **dim; /* [sct] Pointers to full dimension structures */
     int *dmn_id; /* [id] Contiguous vector of dimension IDs */
@@ -1677,9 +1670,17 @@ extern "C" {
 
     double **shp;    /* array of points size [crn_nbr][NBR] */
     double *dp_xyz;  /* maybe useful for 3D stuff */
-
   } poly_sct;
-  
+
+  typedef struct{
+    int src_id;
+    int dst_id;
+    double area;
+    double wgt;
+    double dist;
+  } wgt_sct;
+
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* !__cplusplus */
