@@ -1,15 +1,13 @@
-/* $Header$ */
+ /* $Header$ */
 
 /* Purpose: Calendar utilities */
 
 /* Copyright (C) 1995--present Charlie Zender
    This file is part of NCO, the netCDF Operators. NCO is free software.
    You may redistribute and/or modify NCO under the terms of the 
-   GNU General Public License (GPL) Version 3 with exceptions described in the LICENSE file */
+   3-Clause BSD License with exceptions described in the LICENSE file */
 
 #include "nco_cln_utl.h" /* Calendar utilities */
-#include "libnco.h"
-#include "nco.h"
 
 /* Arrays to hold calendar type units */
 /* Format: year, month, day, hour, minute, second, origin, offset */
@@ -137,7 +135,7 @@ nco_newdate /* [fnc] Compute date a specified number of days from input date */
   } /* end if */
   
   return newdate_YYMMDD;
-} /* end nco_newdate() */
+} /* !nco_newdate() */
 
 tm_typ /* O [enm] Units type */
 nco_cln_get_tm_typ /* Returns time unit type or tm_void if not found */
@@ -164,8 +162,7 @@ nco_cln_get_tm_typ /* Returns time unit type or tm_void if not found */
   
   if(lcl_sng) lcl_sng=(char *)nco_free(lcl_sng);
   return rcd_typ;
-} /* end nco_cln_get_tm_typ() */
-
+} /* !nco_cln_get_tm_typ() */
 
 char *                 /* O [sng] contains newly malloced output string */
 nco_cln_fmt_dt         /*   [fnc] format an output string */
@@ -178,44 +175,37 @@ int ifmt)              /* I [enm] nco_fmt_dt */
 
  buff=(char*)nco_malloc( sizeof(char) * NCO_MAX_LEN_FMT_SNG );
 
- switch(ifmt)
- {
-    /* plain format all out */
-    case fmt_dt_rgl:
-     sprintf(buff,"%04d-%02d-%02d %02d:%02d:%f", ttx->year,ttx->month, ttx->day,ttx->hour,ttx->min,ttx->sec  );
-     break;
-
-
-     /* plain format all out with 'T' char as spacer*/
-   case fmt_dt_iso8601:
-     sprintf(buff,"%04d-%02d-%02dT%02d:%02d:%f", ttx->year,ttx->month, ttx->day,ttx->hour,ttx->min,ttx->sec  );
-     break;
-
-     /* do date and time if time not all zero */
-    case fmt_dt_sht:
-    case fmt_dt_nil:
-      sprintf(bdate,"%04d-%02d-%02d", ttx->year,ttx->month, ttx->day);
-      if( ttx->hour !=0 || ttx->min!=0 || ttx->sec !=0.0 )
-      {
-        int isec;
-        double m_sec, frac_sec;
-
-        frac_sec=modf(ttx->sec, &m_sec);
-        isec=(int)m_sec;
-
-        if( frac_sec==0.0)
-          sprintf(btime, " %02d:%02d:%02d", ttx->hour,ttx->min, isec );
-        else
-          sprintf(btime, " %02d:%02d:%02.7f", ttx->hour,ttx->min, ttx->sec );
-      }
-      sprintf(buff,"%s%s", bdate,btime);
-      break;
+ switch(ifmt){
+ case fmt_dt_nil:
+   buff[0]='\0';
+   break;
+   /* Plain format */
+ case fmt_dt_rgl:
+   sprintf(buff,"%04d-%02d-%02d %02d:%02d:%f", ttx->year,ttx->month, ttx->day,ttx->hour,ttx->min,ttx->sec);
+   break;
+   /* Plain format all out with 'T' char as date-time spacer */
+ case fmt_dt_iso8601:
+   sprintf(buff,"%04d-%02d-%02dT%02d:%02d:%f", ttx->year,ttx->month, ttx->day,ttx->hour,ttx->min,ttx->sec);
+   break;
+   /* Print date and time if time not all zero */
+ case fmt_dt_sht:
+   sprintf(bdate,"%04d-%02d-%02d",ttx->year,ttx->month,ttx->day);
+   if(ttx->hour !=0 || ttx->min!=0 || ttx->sec !=0.0){
+     int isec;
+     double m_sec,frac_sec;
+     
+     frac_sec=modf(ttx->sec, &m_sec);
+     isec=(int)m_sec;
+     
+     if(frac_sec == 0.0) sprintf(btime, " %02d:%02d:%02d", ttx->hour,ttx->min, isec ); else sprintf(btime, " %02d:%02d:%02.7f", ttx->hour,ttx->min, ttx->sec);
+   }
+   sprintf(buff,"%s%s", bdate,btime);
+   break;
  }
-
-   return buff;
+ 
+ return buff;
 }
 
-
 nco_cln_typ /* [enm] Calendar type */
 nco_cln_get_cln_typ /* [fnc] Determine calendar type or cln_nil if not found */
 (const char *ud_sng) /* I [ptr] Units string */
@@ -246,7 +236,7 @@ nco_cln_get_cln_typ /* [fnc] Determine calendar type or cln_nil if not found */
   lcl_sng=(char *)nco_free(lcl_sng);
   
   return rcd_typ;
-} /* end nco_cln_get_cln_typ() */
+} /* !nco_cln_get_cln_typ() */
 
 int /* O [nbr] Number of days */
 nco_cln_days_in_year_prior_to_given_month /* [fnc] Number of days in year prior to month */
@@ -279,8 +269,7 @@ nco_cln_days_in_year_prior_to_given_month /* [fnc] Number of days in year prior
   for(idx=0;idx<mth_idx;idx++) idays+=days[idx];
   
   return idays;
-} /* end nco_cln_days_in_year_prior_to_given_month() */
-
+} /* !nco_cln_days_in_year_prior_to_given_month() */
 
 void
 nco_cln_pop_tm         /* [fnc] Calculate other members  in cln_sct from value*/
@@ -294,8 +283,7 @@ nco_cln_pop_tm         /* [fnc] Calculate other members  in cln_sct from value*/
   double m_value;
   double *data=NULL_CEWI;
 
-  switch(cln_sct->sc_cln)
-  {
+  switch(cln_sct->sc_cln){
     case cln_360:
       data=DATA_360;
       days_per_month=DAYS_PER_MONTH_360;
@@ -318,10 +306,7 @@ nco_cln_pop_tm         /* [fnc] Calculate other members  in cln_sct from value*/
       break;
   } /* end switch */
 
-
   /* take account of origin */
-
-
   /* get integer value and fraction part - nb add origin */
   fr_value=modf(cln_sct->value+(double)data[6], &m_value);
 
@@ -342,20 +327,15 @@ nco_cln_pop_tm         /* [fnc] Calculate other members  in cln_sct from value*/
   days+=1;
   cln_sct->month=1;
 
-  for(idx=0 ; idx<12 ;idx++  )
-  {
-    if( days - days_per_month[idx] <=0 )
-      break;
+  for(idx=0;idx<12;idx++){
+    if(days - days_per_month[idx] <= 0) break;
 
-    days -= days_per_month[idx];
-      ++cln_sct->month;
+    days-=days_per_month[idx];
+    ++cln_sct->month;
   }
   cln_sct->day=days;
-
-
   return;
-} /* end nco_cln_pop_tm() */
-
+} /* !nco_cln_pop_tm() */
 
 void
 nco_cln_pop_val /* [fnc] Calculate value in cln_sct */ 
@@ -406,10 +386,7 @@ nco_cln_pop_val /* [fnc] Calculate value in cln_sct */
   } /* end switch */
   
   return;
-} /* end nco_cln_pop_val() */
-
-
-
+} /* !nco_cln_pop_val() */
 
 double /* O [dbl] time in (base) seconds of tm_typ */
 nco_cln_val_tm_typ
@@ -460,8 +437,7 @@ nco_cln_val_tm_typ
   } /* end switch */ 
   
   return scl;
-} /* end nco_cln_typ_val() */
-
+} /* !nco_cln_typ_val() */
 
 void
 nco_cln_prn_tm         /* [fnc] print tm sct*/
@@ -484,7 +460,134 @@ nco_cln_chk_tm /* [fnc] Is string a UDUnits-compatible calendar format, e.g., "P
   /* Does string contain date keyword? */
   if(strcasestr(unit_sng," from ") || strcasestr(unit_sng," since ") || strcasestr(unit_sng," after ")) return True; else return False;
 
-} /* end nco_cln_chk_tm() */
+} /* !nco_cln_chk_tm() */
+
+int /* O [rcd] Return code */
+nco_clm_nfo_to_tm_bnds /* [fnc] Compute and return climatological time and bounds arrays */
+(int yr_srt, /* I [yr] Year at climo start */
+ int yr_end, /* I [yr] Year at climo start */
+ int mth_srt, /* I [mth] Month at climo start [1..12] format */
+ int mth_end, /* I [mth] Month at climo end [1..12] format */
+ int tpd, /* I [nbr] Timesteps per day [0=monthly, 1=daily, 2, 3, 4, 6, 8, 12, 24=hourly, ...] */
+ const char *unt_sng, /* I [sng] Units of time coordinate (UDUnits format) */
+ const char *cln_sng, /* I [sng] Calendar string of time coordinate (UDUnits format, NULL=none) */
+ double *bnd_val, /* O [frc] Climatology bounds variable values */
+ double *tm_val) /* O [frc] Time coordinate values */
+{
+  const char fnc_nm[]="nco_clm_nfo_to_tm_bnds()"; /* [sng] Function name */
+
+  char srt_sng[200];
+  char end_sng[200];
+  char md_sng[200];
+  
+  int idx;
+  int day_mid=15;
+
+  double step;
+  
+  nco_cln_typ cln_typ;
+  var_sct *var_tmp=NULL_CEWI;
+  
+  step=24.0/tpd;
+
+  if(!unt_sng) (void)fprintf(stdout,"%s: WARNING %s called with empty units string\n",nco_prg_nm_get(),fnc_nm);
+  if(!cln_sng){
+    (void)fprintf(stdout,"%s: WARNING %s called with empty calendar string, will adopt default calendar \"noleap\"\n",nco_prg_nm_get(),fnc_nm);
+    /* Default to noleap (365-day) calendar used by most ESMs */
+    if(!cln_sng) cln_sng=(char *)strdup("noleap");
+  } /* !cln_sng */
+  cln_typ=nco_cln_get_cln_typ(cln_sng);
+  
+  if(cln_typ == cln_nil) return NCO_ERR;
+  
+  sprintf(srt_sng,"seconds since %d-%d-01",yr_srt,mth_srt);
+  
+  /* Move end forward one month */
+  if(++mth_end == 13){
+    mth_end=1;
+   yr_end++;
+  }
+
+  sprintf(end_sng,"seconds since %d-%d-01",yr_end,mth_end);
+  
+  if(tpd == 0){
+
+    if(tm_val){
+
+      tm_val[0]=0.0;
+
+      /* find day in middle of month - see day_mid  */
+      sprintf(md_sng,"seconds since %d-%d-%d",yr_srt,mth_srt, day_mid);
+
+      if(nco_cln_clc_dbl_var_dff(md_sng,unt_sng,cln_typ,&tm_val[0],(var_sct *)NULL) != NCO_NOERR)
+        return NCO_ERR;
+
+    }
+
+    if(bnd_val){
+      bnd_val[0]=0.0;
+      bnd_val[1]=0.0;
+
+      if(nco_cln_clc_dbl_var_dff(srt_sng,unt_sng,cln_typ,&bnd_val[0],(var_sct *)NULL) != NCO_NOERR)
+        return NCO_ERR;
+
+      if(nco_cln_clc_dbl_var_dff(end_sng,unt_sng,cln_typ,&bnd_val[1],(var_sct *)NULL) != NCO_NOERR)
+        return NCO_ERR;
+    } /* !bnd_val */
+
+    return NCO_NOERR;
+  } /* tpd != 0 */
+
+  /* tpd >= 1 so times are daily or diurnal */
+  var_tmp=(var_sct *)nco_malloc(sizeof(var_sct));
+  var_dfl_set(var_tmp);
+  var_tmp->type=NC_DOUBLE;
+
+  if(tm_val){
+     var_tmp->sz=tpd;
+     for(idx=0;idx<tpd;idx++)
+       tm_val[idx]=(step/2 + step*idx)*3600;
+
+     cast_void_nctype(NC_DOUBLE,&var_tmp->val);
+     var_tmp->val.dp=tm_val;
+     cast_nctype_void(NC_DOUBLE,&var_tmp->val);
+
+     if(nco_cln_clc_dbl_var_dff(srt_sng,unt_sng,cln_typ,(double *)NULL,var_tmp) != NCO_NOERR)
+       return NCO_ERR;
+
+     var_tmp->val.vp=NULL;
+  } /* !tm_val */
+
+  if(bnd_val){
+    /* Seconds difference between srt_sng and end_sng */
+    double srt_end_dff;
+
+    if(nco_cln_clc_dbl_var_dff(end_sng,srt_sng,cln_typ,&srt_end_dff,(var_sct *)NULL) != NCO_NOERR)
+      return NCO_ERR;
+
+    /* Start at midnight of previous day (last day of previous month) */
+    srt_end_dff=srt_end_dff - (24.0-step)*3600.0;
+
+    var_tmp->sz=tpd*2;
+
+    for(idx=0;idx<tpd;idx++){
+      bnd_val[2 * idx] = (step * idx) * 3600;
+      bnd_val[2 * idx + 1] = bnd_val[2*idx]+srt_end_dff;
+    }
+
+    cast_void_nctype(NC_DOUBLE,&var_tmp->val);
+    var_tmp->val.dp=bnd_val;
+    cast_nctype_void(NC_DOUBLE,&var_tmp->val);
+
+    if(nco_cln_clc_dbl_var_dff(srt_sng,unt_sng,cln_typ,(double *)NULL,var_tmp) != NCO_NOERR) return NCO_ERR;
+
+    var_tmp->val.vp=NULL;
+  } /* !bnd_val */
+
+  if(var_tmp) var_tmp=nco_var_free(var_tmp);
+
+  return NCO_NOERR;
+} /* !nco_clm_nfo_to_tm_bnds() */
 
 #ifndef ENABLE_UDUNITS
 /* Stub functions to compile without UDUNITS2 */
@@ -500,7 +603,7 @@ var_sct *var)           /* I/O [var_sct] var values modified */
   (void)fprintf(stderr,"%s: WARNING NCO was built without UDUnits. NCO is therefore unable to convert values from \"%s\" to \"%s\". See http://nco.sf.net/nco.html#rbs for more information.\n%s: HINT Re-build or re-install NCO enabled with UDUnits.\n",nco_prg_nm_get(),fl_unt_sng,fl_bs_sng,nco_prg_nm_get());
 
   return NCO_NOERR;
-} /* end nco_cln_clc_dbl_var_dff() */
+} /* !nco_cln_clc_dbl_var_dff() */
 
 int /* [flg] NCO_NOERR or NCO_ERR */ 
 nco_cln_clc_dbl_org(   /* [fnc] difference between two co-ordinate units */
@@ -524,7 +627,7 @@ nco_cln_sng_rbs /* [fnc] Rebase calendar string for legibility */
 {
   lgb_sng[0]='\0'; /* CEWI */
   return NCO_NOERR;
-} /* end nco_cln_sng_rbs() */
+} /* !nco_cln_sng_rbs() */
 
 int
 nco_cln_var_prs
@@ -553,7 +656,16 @@ nco_cln_prs_tm /* UDUnits2 Extract time stamp from parsed UDUnits string */
 #ifdef ENABLE_UDUNITS
 # ifdef HAVE_UDUNITS2_H
 
-/* UDUnits2 routines */
+
+
+
+
+
+/* Make a udunits2 converter:
+ *
+ * Used internally by nco_cln_clc_dbl_dff() and  nco_cln_clc_var_dff()
+ * NB it is up to the calling function to free up the converter after use */
+
 cv_converter*   /* UDUnits converter */
 nco_cln_cnv_mk  /* [fnc] UDUnits2 create a custom converter  */
 (const char *fl_unt_sng, /* I [ptr] units attribute string from disk */
@@ -616,6 +728,16 @@ nco_cln_cnv_mk  /* [fnc] UDUnits2 create a custom converter  */
   return ut_cnv;
 }  /* end UDUnits2 nco_cln_cnv_mk() */
 
+
+/* fl_unt_sng - source units
+ * fl_bs_sng  - target units
+ * *og_val    -  single value of type NC_DOUBLE
+ *
+ * Conversion done by udunits library.
+ * Input and Output value og_val
+ */
+
+
 int /* [flg] NCO_NOERR or NCO_ERR */ 
 nco_cln_clc_dbl_dff( /* [fnc] difference between two co-ordinate units */
 const char *fl_unt_sng, /* I [ptr] units attribute string from disk */
@@ -625,18 +747,27 @@ double *og_val)
   cv_converter *ut_cnv=NULL; 
 
   /* Do nothing if units identical */
-  if(strcasecmp(fl_unt_sng,fl_bs_sng)==0) return NCO_NOERR;
+  if(strcasecmp(fl_unt_sng,fl_bs_sng) == 0) return NCO_NOERR;
 
   /* Convert */
   ut_cnv=nco_cln_cnv_mk(fl_unt_sng, fl_bs_sng);
 
   if(ut_cnv != NULL) og_val[0]=cv_convert_double(ut_cnv,og_val[0]); else return NCO_ERR;     
 
-  cv_free(ut_cnv);  
+  cv_free(ut_cnv);
 
   return NCO_NOERR;          
 } /* end UDUnits2 nco_cln_clc_dbl_dff() */
 
+
+/* fl_unt_sng - source units
+ * fl_bs_sng  - target units
+ * var        -  var_sct array of values.
+ *
+ * Conversion done by udunits library
+ * if var->type less than NC_FLOAT then var is promoted to NC_DOUBLE for conversion then
+ * demoted back afterwards. */
+
 int /* [flg] NCO_NOERR or NCO_ERR */ 
 nco_cln_clc_var_dff /* [fnc] difference between two co-ordinate units */
 (const char *fl_unt_sng, /* I [ptr] units attribute string from disk */
@@ -647,9 +778,11 @@ nco_cln_clc_var_dff /* [fnc] difference between two co-ordinate units */
 
   ptr_unn op1;    
 
-  size_t idx;    
+  size_t idx;
 
   size_t sz;
+
+  nc_type var_typ_org;
   
   /* Do nothing if units identical */
   if(!strcasecmp(fl_unt_sng,fl_bs_sng)) return NCO_NOERR;
@@ -659,7 +792,11 @@ nco_cln_clc_var_dff /* [fnc] difference between two co-ordinate units */
 
   if(ut_cnv == NULL) return NCO_ERR;     
 
-  sz=var->sz;  
+  sz=var->sz;
+
+  var_typ_org=var->type;
+  if(!(var->type==NC_FLOAT && var->type==NC_DOUBLE))
+    var=nco_var_cnf_typ(NC_DOUBLE,var);
 
   (void)cast_void_nctype(var->type,&op1);                
   op1=var->val;
@@ -692,9 +829,37 @@ nco_cln_clc_var_dff /* [fnc] difference between two co-ordinate units */
  cv_free(ut_cnv);  
  (void)cast_nctype_void(var->type,&op1);
 
+ /* convert back to original type */
+ var=nco_var_cnf_typ(var_typ_org,var);
+
  return NCO_NOERR;
 } /* !nco_cln_clc_var_dff() */
 
+
+
+
+/* This function is similar to nco_cln_clc_dbl_org() except the value(s) to be converted
+   is in og_val or an array of values in var.
+
+   fl_unt_sng - is the  source units string
+   fl_bs_sng  - is the targets unit string
+   lmt_cln    - is the calendar type or lmt_nil (as approriate)
+   *og_val    - input value or (double*)NULL if var is used
+   *var       - input array of values or (var_sct*)NULL if og_val used.
+   *
+   * if a non standard lmt_cln is used.then the nco_cln_utl is used (transparently) to do the conversion
+   * otherwise the udunits2 library is used.
+   *
+   * If var type is less than NC_FLOAT or NC_DOUBLE. Then var is promoted to NC_DOUBLE for the conversion;
+   * then demoted back to original type after.
+   *
+   *
+   * Examples of usage:
+   * nco_cln_clc_dbl_var_dff("seconds since 2001-01-01", "days since 1978-02-04", cln_365, og_val, (var_sct*)NULL);
+   *
+   * nco_cln_clc_dbl_var_dff("days since 1900-01-01", "days since 1950-01-31", cln_std, (double*)NULL, var);
+   */
+
 int /* [flg] NCO_NOERR or NCO_ERR */ 
 nco_cln_clc_dbl_var_dff( /* [fnc] difference between two co-ordinate units */
 const char *fl_unt_sng, /* I [ptr] units attribute string from disk */
@@ -730,7 +895,28 @@ var_sct *var) /* I/O [var_sct] var values modified - can be NULL  */
   return rcd;
 } /* end UDUnits2 nco_cln_clc_dbl_var_dff() */
 
-int   /* [flg] NCO_NOERR or NCO_ERR */ 
+
+
+/* If you wish to replicate the function of the udunits2 command line application then
+ * This should be your first port of call.
+ * val_unt_sng takes the form  "value  units" - where val is interpreted as a double
+ * fl_bs_sng is of the form "units" and is the target units to be converted to
+ *
+ * Examples of usage:
+ * "10 days since 2001-01-01"  "hours since 2001-01-09"  - 48 (hours)
+ * "5 minutes since 1992-02-29T12:00:00" "seconds since 1992-01-31"  2.5491e+06 (seconds)
+ * "1 mile" "m"    1609.34 (m)
+ * "12 inches" "cm" 30.48 cm
+ *
+ * If lmt_cln is empty or of type "standard" (cln_std) or "julian" (cln_jul) or "gregorian" (cln_grg) then
+ * the conversion is done by the udunits2 library.
+ *
+ * if lmt_cln is of type "360_day" (cln_360) or "365_day" (cln_365) or "366_day" (cln_366) then
+ * the conversion is done "transparently" by the nco_cln_utl library.
+ */
+
+
+int   /* [flg] NCO_NOERR or NCO_ERR */
 nco_cln_clc_dbl_org(   /* [fnc] difference between two co-ordinate units */
 const char *val_unt_sng, /* I [ptr] input value and  units in the same string */
 const char *fl_bs_sng,  /* I [ptr] units attribute string from disk */
@@ -780,7 +966,7 @@ double *og_val)         /* O [dbl] output value */
   }else rcd=nco_cln_clc_dbl_dff(lcl_unt_sng,fl_bs_sng,&val_dbl);
 
   /* Copy over iff successful */ 
-  if(rcd==NCO_NOERR) *og_val=val_dbl; else (void)fprintf(stderr,"%s: ERROR %s: report unt_sng=%s bs_sng=%s calendar=%d og_val=%f\n",nco_prg_nm_get(),fnc_nm,val_unt_sng,fl_bs_sng,lmt_cln, val_dbl);  
+  if(rcd == NCO_NOERR) *og_val=val_dbl; else (void)fprintf(stderr,"%s: ERROR %s: report unt_sng=%s bs_sng=%s calendar=%d og_val=%f\n",nco_prg_nm_get(),fnc_nm,val_unt_sng,fl_bs_sng,lmt_cln, val_dbl);  
  
   return rcd;        
 } /* !nco_cln_clc_dbl_org() */
@@ -789,9 +975,9 @@ int /* [rcd] Successful conversion returns NCO_NOERR */
 nco_cln_clc_tm /* [fnc] Difference between two coordinate units */
 (const char *fl_unt_sng, /* I [ptr] Units attribute string from disk */
  const char *fl_bs_sng, /* I [ptr] Units attribute string from disk */
- nco_cln_typ lmt_cln, /* [enum] Calendar type of coordinate variable */ 
+ nco_cln_typ lmt_cln, /* [enum] Calendar type of coordinate variable */
  double *og_val, /* I/O [ptr] */
- var_sct *var) /* I/O [ptr] */  
+ var_sct *var) /* I/O [ptr] */
 {
   /* Called for target units in fl_bs_sng of form "value unit since date-stamp" for cln_360 or cln_365
      Either "var" is NULL and there is a single value to process *og_val or var is initialized and og_val is NULL */
@@ -802,12 +988,12 @@ nco_cln_clc_tm /* [fnc] Difference between two coordinate units */
   double crr_val=0.0;
   double scl_val=1.0;
 
-  tm_typ unt_tm_typ; /* enum for units type in fl_unt_sng */  
+  tm_typ unt_tm_typ; /* enum for units type in fl_unt_sng */
   tm_typ bs_tm_typ; /* enum for units type in fl_bs_sng */
 
   tm_cln_sct unt_cln_sct;
   tm_cln_sct bs_cln_sct;
-  
+
   /* Die if unsupported calendar type */
   if(lmt_cln != cln_360 && lmt_cln != cln_365 && lmt_cln != cln_366){
     (void)fprintf(stderr,"%s: %s reports invalid calendar type lmt_cln=%d. Only cln_365,cln_360 cln_366 allowed.\n",nco_prg_nm_get(),fnc_nm,lmt_cln);
@@ -817,35 +1003,35 @@ nco_cln_clc_tm /* [fnc] Difference between two coordinate units */
   /* Obtain units type from fl_bs_sng */
   tmp_sng=(char *)nco_calloc(NCO_MAX_LEN_TMP_SNG,sizeof(char));
   if(sscanf(fl_bs_sng,"%s",tmp_sng) != 1) return NCO_ERR;
-  bs_tm_typ=nco_cln_get_tm_typ(tmp_sng);  
+  bs_tm_typ=nco_cln_get_tm_typ(tmp_sng);
   if(nco_dbg_lvl_get() >= nco_dbg_crr) (void)fprintf(stderr,"%s: %s reports unt_sng=\"%s\", bs_sng=\"%s\", tmp_sng=\"%s\"\n",nco_prg_nm_get(),fnc_nm,fl_unt_sng,fl_bs_sng,tmp_sng);
   if(tmp_sng) tmp_sng=(char *)nco_free(tmp_sng);
 
-  /* Is unit string a bare date string? */ 
+  /* Is unit string a bare date string? */
   tmp_sng=(char *)nco_calloc(NCO_MAX_LEN_TMP_SNG,sizeof(char));
   if(!strncmp("s@",fl_unt_sng,2)) unt_tm_typ=bs_tm_typ;
-  else if(sscanf(fl_unt_sng,"%s",tmp_sng) == 1) unt_tm_typ=nco_cln_get_tm_typ(tmp_sng);  
+  else if(sscanf(fl_unt_sng,"%s",tmp_sng) == 1) unt_tm_typ=nco_cln_get_tm_typ(tmp_sng);
   else return NCO_ERR;
   if(tmp_sng) tmp_sng=(char *)nco_free(tmp_sng);
-  
-  /* Assume non-standard calendar */ 
+
+  /* Assume non-standard calendar */
   if(nco_cln_prs_tm(fl_unt_sng,&unt_cln_sct) == NCO_ERR) return NCO_ERR;
   if(nco_cln_prs_tm(fl_bs_sng,&bs_cln_sct) == NCO_ERR) return NCO_ERR;
-  
+
   unt_cln_sct.sc_typ=bs_tm_typ;
   bs_cln_sct.sc_typ=bs_tm_typ;
-  
+
   unt_cln_sct.sc_cln=lmt_cln;
-  bs_cln_sct.sc_cln=lmt_cln; 
+  bs_cln_sct.sc_cln=lmt_cln;
   (void)nco_cln_pop_val(&unt_cln_sct);
   (void)nco_cln_pop_val(&bs_cln_sct);
-  
+
   /* Get offset */
-  crr_val=(unt_cln_sct.value-bs_cln_sct.value)/nco_cln_val_tm_typ(lmt_cln,bs_tm_typ);                 
+  crr_val=(unt_cln_sct.value-bs_cln_sct.value)/nco_cln_val_tm_typ(lmt_cln,bs_tm_typ);
 
   /* Scale factor */
   if(unt_tm_typ == bs_tm_typ) scl_val=1.0; else scl_val=nco_cln_val_tm_typ(lmt_cln,unt_tm_typ)/nco_cln_val_tm_typ(lmt_cln,bs_tm_typ);
-  
+
   if(nco_dbg_lvl_get() >= nco_dbg_crr){
     nco_cln_prn_tm(&unt_cln_sct);
     nco_cln_prn_tm(&bs_cln_sct);
@@ -854,46 +1040,59 @@ nco_cln_clc_tm /* [fnc] Difference between two coordinate units */
     (void)fprintf(stderr,"\n");
   } /* !dbg */
 
+
+
+
   if(og_val){
     *og_val=(*og_val)*scl_val+crr_val;
   }else if(var){
     size_t sz;
     size_t idx;
     ptr_unn op1;
-    
-    sz=var->sz;  
+    /* save  original type if have to promote type to do conversion */
+    nc_type var_typ_org;
+
+    var_typ_org=var->type;
+
+    /* promote type to do conversion */
+    if( var->type!=NC_FLOAT ||  var->type !=NC_DOUBLE)
+      var=nco_var_cnf_typ(NC_DOUBLE, var);
+
+
+    sz=var->sz;
     op1=var->val;
     (void)cast_void_nctype(var->type,&op1);
 
     if(var->type == NC_DOUBLE){
       double *dp;
       dp=op1.dp;
-      if(var->has_mss_val){  
-	double mss_dbl=var->mss_val.dp[0]; 
+      if(var->has_mss_val){
+	double mss_dbl=var->mss_val.dp[0];
 	for(idx=0;idx<sz;idx++)
-	  if(dp[idx] != mss_dbl) dp[idx]=dp[idx]*scl_val+crr_val; 
+	  if(dp[idx] != mss_dbl) dp[idx]=dp[idx]*scl_val+crr_val;
       }else{
-	for(idx=0;idx<sz;idx++) dp[idx]=dp[idx]*scl_val+crr_val; 
+	for(idx=0;idx<sz;idx++) dp[idx]=dp[idx]*scl_val+crr_val;
       } /* !var->has_mss_val */
     } /* !NC_DOUBLE */
 
     if(var->type == NC_FLOAT){
       float *fp;
       fp=op1.fp;
-      if(var->has_mss_val){  
-	float mss_flt=var->mss_val.fp[0]; 
+      if(var->has_mss_val){
+	float mss_flt=var->mss_val.fp[0];
 	for(idx=0;idx<sz;idx++)
-	  if(fp[idx] != mss_flt) fp[idx]=fp[idx]*(float)scl_val+(float)crr_val;                      
+	  if(fp[idx] != mss_flt) fp[idx]=fp[idx]*(float)scl_val+(float)crr_val;
       }else{
-	for(idx=0;idx<sz;idx++) fp[idx]=fp[idx]*(float)scl_val+(float)crr_val;                      
+	for(idx=0;idx<sz;idx++) fp[idx]=fp[idx]*(float)scl_val+(float)crr_val;
       } /* !has_mss_val */
     } /* !NC_FLOAT */
    (void)cast_nctype_void(var->type,&op1);
-  } /* !var */
-  
-  return NCO_NOERR;
 
-} /* end nco_cln_clc_tm() */
+    var=nco_var_cnf_typ(var_typ_org, var);
+  } /* !var */
+
+  return NCO_NOERR;
+} /* !nco_cln_clc_tm() */
 
 int /* [rcd] Successful conversion returns NCO_NOERR */
 nco_cln_prs_tm /* UDUnits2 Extract time stamp from parsed UDUnits string */
@@ -1058,9 +1257,7 @@ nco_cln_sng_rbs /* [fnc] Rebase calendar string for legibility */
   lgb_sng[0]='\0'; /* CEWI */
 
   return NCO_NOERR;
-
-} /* end nco_cln_sng_rbs() */
-
+} /* !nco_cln_sng_rbs() */
 
 int
 nco_cln_var_prs
@@ -1068,8 +1265,7 @@ nco_cln_var_prs
  nco_cln_typ lmt_cln,
  int ifmt,
  var_sct *var,
- var_sct *var_ret
-)
+ var_sct *var_ret)
 {
   size_t sz;
   size_t idx;
@@ -1077,9 +1273,6 @@ nco_cln_var_prs
 
   double resolution;
   tm_cln_sct tm;
-
-
-
   /* base units for udunits */
   const char *bs_sng="seconds since 2001-01-01";
   const char *fnc_nm="nco_cln_var_prs";
@@ -1093,102 +1286,71 @@ nco_cln_var_prs
   if(nco_dbg_lvl_get() >= nco_dbg_crr)
      (void)fprintf(stderr,"%s: %s reports unt_sng=%s bs_sng=%s calendar=%d\n",nco_prg_nm_get(),fnc_nm,fl_unt_sng,bs_sng,lmt_cln);
 
-  /* rebase to seconds since blah-blah */
-  if(nco_cln_clc_dbl_var_dff(fl_unt_sng,bs_sng,lmt_cln, (double*)NULL, var ) != NCO_NOERR )
-     return NCO_ERR;
-
-
+  /* Rebase to seconds since blah-blah */
+  if(nco_cln_clc_dbl_var_dff(fl_unt_sng,bs_sng,lmt_cln, (double*)NULL, var ) != NCO_NOERR) return NCO_ERR;
 
   cast_void_nctype(var->type,&var->val);
 
-  if(var_ret->type !=NC_STRING)
-      nco_var_cnf_typ(NC_STRING, var_ret);
+  if(var_ret->type !=NC_STRING) nco_var_cnf_typ(NC_STRING, var_ret);
 
-  if( var_ret->val.vp)
-      var_ret->val.vp=(void*)nco_free(var_ret->val.vp);
+  if(var_ret->val.vp) var_ret->val.vp=(void*)nco_free(var_ret->val.vp);
 
   var_ret->val.vp=nco_malloc( sizeof(nco_string) *var_ret->sz);
 
-
   var_ret->has_mss_val=True;
   var_ret->mss_val.vp=nco_malloc(sizeof(nco_string*));
 
   cast_void_nctype(var_ret->type,&var_ret->val);
 
   var_ret->mss_val.sngp[0]=strdup(empty_sng);
-
-
   sz=var->sz;
-
   tm.sc_cln=lmt_cln;
-
-
   // (void)fprintf(stderr,"%s: %s reports var \"%s\" has missing value %d\n",nco_prg_nm_get(),fnc_nm,var->nm,var->has_mss_val);
 
-
-
-  if(var->type == NC_DOUBLE) {
+  if(var->type == NC_DOUBLE){
     double mss_val_dbl;
-    if(var->has_mss_val)
-       mss_val_dbl=var->mss_val.dp[0];
-
-    for (idx = 0; idx < sz; idx++) {
-
-
-      if(var->has_mss_val && var->val.dp[idx]==mss_val_dbl) {
+    if(var->has_mss_val) mss_val_dbl=var->mss_val.dp[0];
+    for (idx = 0; idx < sz; idx++){
+      if(var->has_mss_val && var->val.dp[idx]==mss_val_dbl){
         var_ret->val.sngp[idx] = strdup(empty_sng);
         continue;
       }
 
       tm.value = var->val.dp[idx];
-
       if (lmt_cln == cln_360 || lmt_cln == cln_365 || lmt_cln == cln_366)
         nco_cln_pop_tm(&tm);
       else
         (void) ut_decode_time(tm.value, &tm.year, &tm.month, &tm.day, &tm.hour, &tm.min, &tm.sec, &resolution);
 
       var_ret->val.sngp[idx] = nco_cln_fmt_dt(&tm, ifmt);
-
     }
-  }
-  else if(var->type==NC_FLOAT){
+  }else if(var->type==NC_FLOAT){
 
     float mss_val_flt;
-    if(var->has_mss_val)
-       mss_val_flt=var->mss_val.fp[0];
+    if(var->has_mss_val) mss_val_flt=var->mss_val.fp[0];
 
     for (idx = 0; idx < sz; idx++) {
-
       if(var->has_mss_val && var->val.fp[idx]==mss_val_flt ){
         var_ret->val.sngp[idx] = strdup(empty_sng);
         continue;
       }
 
+      tm.value=(double)(var->val.fp[idx]);
 
-      tm.value = (double) (var->val.fp[idx]);
-
-      if (lmt_cln == cln_360 || lmt_cln == cln_365 || lmt_cln == cln_366)
+      if(lmt_cln == cln_360 || lmt_cln == cln_365 || lmt_cln == cln_366)
         nco_cln_pop_tm(&tm);
       else
-        (void) ut_decode_time(tm.value, &tm.year, &tm.month, &tm.day, &tm.hour, &tm.min, &tm.sec, &resolution);
-
+        (void)ut_decode_time(tm.value, &tm.year, &tm.month, &tm.day, &tm.hour, &tm.min, &tm.sec, &resolution);
       var_ret->val.sngp[idx] = nco_cln_fmt_dt(&tm, ifmt);
-
     }
-
   }
 
-
   cast_nctype_void(var->type,&var->val);
   cast_nctype_void(var_ret->type,&var->val);
 
-    return NCO_NOERR;
-
-} /* end nco_cln_var_prs() */
-
-
-
+  return NCO_NOERR;
 
+} /* !nco_cln_var_prs() */
 
 # endif /* HAVE_UDUNITS2_H */
 #endif /* ENABLE_UDUNITS */
