@@ -72,7 +72,7 @@ extern "C" {
     double sec;
     double value;
   } tm_cln_sct;
-  
+
   int /* O [nbr] Number of days to end of month */
   nco_nd2endm /* [fnc] Compute number of days to end of month */
   (const int mth, /* I [mth] Month */
@@ -173,6 +173,17 @@ extern "C" {
   (const char *unt_sng, /* I [ptr] units attribute string */  
    tm_cln_sct *tm_in); /*  O [sct] struct to be populated */
 
+  int /* O [rcd] Return code */
+  nco_clm_nfo_to_tm_bnds /* [fnc] Compute and return climatological time and bounds arrays */
+  (int yr_srt, /* I [yr] Year at climo start */
+   int yr_end, /* I [yr] Year at climo start */
+   int mth_srt, /* I [mth] Month at climo start [1..12] format */
+   int mth_end, /* I [mth] Month at climo end [1..12] format */
+   int tpd, /* I [nbr] Timesteps per day [0=monthly, 1=daily, 2, 3, 4, 6, 8, 12, 24=hourly, ...] */
+   const char *unt_sng, /* I [sng] Units of time coordinate (UDUnits format) */
+   const char *cln_sng, /* I [sng] Calendar string of time coordinate (UDUnits format, NULL=none) */
+   double *bnd_val, /* O [frc] Climatology bounds variable values */
+   double *tm_val); /* O [frc] Time coordinate values */
 
 #ifdef ENABLE_UDUNITS
 # ifdef HAVE_UDUNITS2_H
@@ -195,7 +206,6 @@ extern "C" {
    nco_cln_typ lmt_cln,  /* [enm] Calendar type of coordinate var */ 
    double *rgn_val, /* I/O [ptr] time diff in units based on fl_bs_sng */ 
    var_sct *var);   /* I/O [ptr]  */ 
-  
 
 # endif /* !HAVE_UDUNITS2_H */
 #endif /* !ENABLE_UDUNITS */
