@@ -5,7 +5,7 @@
 /* Copyright (C) 1995--present Charlie Zender
    This file is part of NCO, the netCDF Operators. NCO is free software.
    You may redistribute and/or modify NCO under the terms of the 
-   GNU General Public License (GPL) Version 3 with exceptions described in the LICENSE file */
+   3-Clause BSD License with exceptions described in the LICENSE file */
 
 /* Usage:
 #include "nco_netcdf.h" *//* NCO wrappers for netCDF C library */
@@ -51,6 +51,12 @@
 #ifdef NC_HAVE_META_H
 # include <netcdf_meta.h> /* NC_VERSION_..., HAVE_NC_RENAME_GRP */	 
 #endif /* !NC_HAVE_META_H */
+#ifndef NC_LIB_VERSION
+# define NC_LIB_VERSION ( NC_VERSION_MAJOR * 100 + NC_VERSION_MINOR * 10 + NC_VERSION_PATCH )
+#endif /* !NC_LIB_VERSION */
+#if NC_LIB_VERSION >= 474
+# include <netcdf_filter.h> /* netCDF filters */	 
+#endif /* !4.7.4 */
 #ifdef ENABLE_MPI
 # include <mpi.h> /* MPI definitions */
 # include <netcdf_par.h> /* Parallel netCDF definitions */
@@ -59,7 +65,7 @@
 /* Personal headers */
 #include "nco_typ.h" /* Type definitions, opaque types */
 
-/* NB: 
+/* NB:
    nco_netcdf.h does #include nco_typ.h which #defines some forward-compatibility tokens
    nco_netcdf.h does not #include nco.h which #defines some forward-compatibility tokens
    This barrier helps segregate NCO from wrappers
@@ -199,6 +205,42 @@
 #endif
 #ifndef NC_EACCESS
 # define NC_EACCESS      (-77)      /**< Access Failure */
+#endif
+
+  /* Backwards-compatibility error codes implemented in 202004 to workaround netCDF 4.7.4 backwards incompatibility with nc_inq_var_filter()
+     Currently used only in nco_prn.c */
+#ifndef NC_ENOFILTER
+# define NC_ENOFILTER (-136)      /**< Filter not defined on variable. */
+#endif
+
+  /* Many backwards-compatibility error codes implemented in 202008 to diagnose problems with DAP
+     Currently used only in nco_netcdf.c */
+#ifndef NC_EDAP
+# define NC_EDAP         (-66)      /**< Generic DAP error */
+#endif
+#ifndef NC_EDAPSVC
+# define NC_EDAPSVC      (-70)      /**< DAP server error */
+#endif
+#ifndef NC_EDAS
+# define NC_EDAS         (-71)      /**< Malformed or inaccessible DAS */
+#endif
+#ifndef NC_EDDS
+# define NC_EDDS         (-72)      /**< Malformed or inaccessible DDS */
+#endif
+#ifndef NC_EDMR
+# define NC_EDMR         NC_EDDS    /**< Dap4 alias */
+#endif
+#ifndef NC_EDATADDS
+# define NC_EDATADDS     (-73)      /**< Malformed or inaccessible DATADDS */
+#endif
+#ifndef NC_EDATADAP
+# define NC_EDATADAP     NC_EDATADDS    /**< Dap4 alias */
+#endif
+#ifndef NC_EDAPURL
+# define NC_EDAPURL      (-74)      /**< Malformed DAP URL */
+#endif
+#ifndef NC_EDAPCONSTRAINT
+# define NC_EDAPCONSTRAINT (-75)    /**< Malformed DAP Constraint*/
 #endif
 
 /* Some netCDF3 stubs for netCDF4 routines need netCDF4-only return codes
@@ -291,7 +333,7 @@ nco_sng_cnv_err /* [fnc] Print error and exit for failed strtol()-type calls */
 (const char *cnv_sng, /* I [sng] String to convert */
  const char *sng_cnv_fnc, /* I [sng] Name of function used to convert string */
  const char *err_ptr); /* I [chr] First illegal character in string */
-/* end nco_sng_cnv_err() */
+/* !nco_sng_cnv_err() */
 
 void 
 nco_err_exit /* [fnc] Print netCDF error message, routine name, then exit */
@@ -465,6 +507,7 @@ int nco_def_var(const int nc_id,const char * const var_nm,const nc_type var_typ,
 int nco_def_var_chunking(const int nc_id,const int var_id,const int srg_typ,const size_t * const cnk_sz);
 int nco_def_var_deflate(const int nc_id,const int var_id,const int shuffle,const int deflate,const int dfl_lvl);
 int nco_def_var_filter(const int nc_id,const int var_id,const unsigned int flt_id,const size_t prm_nbr,const unsigned int *prm_lst);
+int nco_def_var_filterx(const int nc_id,const int var_id,const char * const flt_nm,const size_t prm_nbr,const char ** const prm_lst);
 int nco_def_var_fletcher32(const int nc_id,const int var_id,const int chk_typ);
 int nco_inq_var(const int nc_id,const int var_id,char * const var_nm,nc_type * const var_typ,int * const dmn_nbr,int * const dmn_id,int * const att_nbr);
 int nco_inq_var_chunking(const int nc_id,const int var_id,int * const srg_typ,size_t * const cnk_sz);
@@ -472,6 +515,10 @@ int nco_inq_var_deflate(const int nc_id,const int var_id,int * const shuffle,int
 int nco_inq_var_endian(const int nc_id,const int var_id,int * const ndn_typ);
 int nco_inq_var_fill(const int nc_id,const int var_id,int * const fll_nil,void * const fll_val);
 int nco_inq_var_filter(const int nc_id,const int var_id,unsigned int * const flt_id,size_t * const prm_nbr,unsigned int * const prm_lst);
+int nco_inq_var_filter_flg(const int nc_id,const int var_id,unsigned int * const flt_id,size_t * const prm_nbr,unsigned int * const prm_lst);
+int nco_inq_var_filterids(const int nc_id,const int var_id,size_t * const flt_nbr,unsigned int * const flt_lst);
+int nco_inq_var_filter_info(const int nc_id,const int var_id,const unsigned int flt_id,size_t * const prm_nbr,unsigned int * const prm_lst);
+int nco_var_filter_remove(const int nc_id,const int var_id,const unsigned int flt_id);
 int nco_inq_var_fletcher32(const int nc_id,const int var_id,int * const chk_typ);
 int nco_inq_var_packing(const int nc_id,const int var_id,int * const packing);
 int nco_inq_vardimid(const int nc_id,const int var_id,int * const dmn_id);
@@ -538,16 +585,26 @@ int nco_get_att(const int nc_id,const int var_id,const char * const att_nm,void
 
   /* 20180923: netcdf_mem.h always contains nc_open_mem()
      nco_create_mem(), nco_open_memio(), nco_close_memio(), NC_memio, and NC_MEMIO_LOCKED introduced in netcdf_mem.h versions 4.6.2 */
-#if NC_LIB_VERSION < 462 
+#if NC_LIB_VERSION < 462
   int nc_create_mem(const char * const fl_nm,const int mode,const size_t sz_ntl,int * const nc_id);
   int nc_open_memio(const char * const fl_nm,const int mode,NC_memio * const info,int * const nc_id);
   int nc_close_memio(const int nc_id,NC_memio * const info);
-#endif /* 4.6.2 */
+#endif /* !4.6.2 */
 
-#if NC_LIB_VERSION < 460 
+#if NC_LIB_VERSION < 460
   int nc_def_var_filter(const int nc_id,const int var_id,const unsigned int flt_id,const size_t prm_nbr,const unsigned int * const prm_lst);
   int nc_inq_var_filter(const int nc_id,const int var_id,unsigned int * const flt_id,size_t * const prm_nbr,unsigned int * const prm_lst);
-#endif /* 4.6.0 */
+#endif /* !4.6.0 */
+
+#if NC_LIB_VERSION < 474
+  int nc_inq_var_filterids(const int nc_id,const int var_id,size_t * const flt_nbr,unsigned int * const flt_lst);
+  int nc_inq_var_filter_info(const int nc_id,const int var_id,const unsigned int flt_id,size_t * const prm_nbr,unsigned int * const prm_lst);
+  int nc_var_filter_remove(const int nc_id,const int var_id,const unsigned int flt_id);
+#endif /* !4.7.4 */
+
+#if NC_LIB_VERSION < 480
+  int nc_def_var_filterx(const int nc_id,const int var_id,const char * const flt_nm,const size_t prm_nbr,const char ** const prm_lst);
+#endif /* !4.8.0 */
 
   /* Begin netCDF4 stubs */
 #ifndef HAVE_NETCDF4_H
