@@ -1,8 +1,14 @@
+/* $Header$ */
+
+/* Purpose: Functions that manipulate lists of polygons */
+
+/* Copyright (C) 2018--present Charlie Zender
+   This file is part of NCO, the netCDF Operators. NCO is free software.
+   You may redistribute and/or modify NCO under the terms of the 
+   3-Clause BSD License with exceptions described in the LICENSE file */
 
 #include "nco_ply_lst.h"
 
-
-/************************ functions that manipulate lists of polygons ****************************************************/
 void
 nco_poly_re_org_lst(  /* for each poly_sct*  in list re-order points so that first point is the leftermost point */
 poly_sct **pl_lst,
@@ -62,11 +68,6 @@ int arr_nbr)
 
 }
 
-
-
-
-
-
 poly_sct **             /* [O] [nbr]  size of array */
 nco_poly_lst_mk(
 double *area, /* I [sr] Area of source grid */
@@ -285,22 +286,16 @@ nco_grd_lon_typ_enm grd_lon_typ) /* I [num]  */
   else
     bwrp=True;
 
-
-
   // printf("About to print poly sct   grd_sz=%d grd_crn_nbr=%d\n", grd_sz, grd_crn_nbr);
   for(idx=0;idx<grd_sz; idx++)
   {
-
     /* check mask and area */
     if( msk[idx]==0 || area[idx] == 0.0 ) {
       pl_lst[idx]= nco_poly_dpl(pl_msk);
       msk_cnt++;
       continue;
-
     }
 
-
-
     pl=nco_poly_init_lst(poly_rll, grd_crn_nbr,0, idx, lon_ptr, lat_ptr);
     lon_ptr+=(size_t)grd_crn_nbr;
     lat_ptr+=(size_t)grd_crn_nbr;
@@ -314,22 +309,18 @@ nco_grd_lon_typ_enm grd_lon_typ) /* I [num]  */
       pl_lst[idx]= nco_poly_dpl(pl_msk);
       msk_cnt++;
       continue;
-
     }
 
     /* add centroid from input  */
     pl->dp_x_ctr=lon_ctr[idx];
     pl->dp_y_ctr=lat_ctr[idx];
 
-
-
     /* pop shp */
     nco_poly_shp_pop(pl);
 
     /* add min max */
     nco_poly_minmax_add(pl, grd_lon_typ, bchk_caps);
 
-
     /* if coords cannot deal with wrapping */
     if( pl->bwrp  && bwrp==False   )
     {
@@ -337,10 +328,8 @@ nco_grd_lon_typ_enm grd_lon_typ) /* I [num]  */
       pl_lst[idx]= nco_poly_dpl(pl_msk);
       msk_cnt++;
       continue;
-
     }
 
-
     /* The area of an RLL grid needs to be re-calculated  as we have to take account of lines of latitude as great circles */
     nco_poly_area_add(pl);
 
@@ -352,9 +341,6 @@ nco_grd_lon_typ_enm grd_lon_typ) /* I [num]  */
     /* simple center of a rll cell - should always be inside of polygon */
     nco_poly_ctr_add(pl, grd_lon_typ);
 
-
-
-
     if(nco_dbg_lvl_get()>= nco_dbg_dev  )
       if(pl->bwrp)
         nco_poly_prn(pl,0);
@@ -367,25 +353,17 @@ nco_grd_lon_typ_enm grd_lon_typ) /* I [num]  */
 
 
     pl_lst[idx]=pl;
-
-
   }
 
-
   if(nco_dbg_lvl_get() >=  nco_dbg_dev )
     (void)fprintf(stderr, "%s: %s size input list(%lu), size output list(%lu)  total area=%.15e  num wrapped= %d num caps=%d num masked=%d\n", nco_prg_nm_get(),fnc_nm, grd_sz, grd_sz, tot_area, wrp_cnt, wrp_y_cnt, msk_cnt);
 
-
-
   pl_msk=nco_poly_free(pl_msk);
 
   return pl_lst;
 
 }
 
-
-
-
 poly_sct **             /* [O] [nbr]  size of array */
 nco_poly_lst_mk_sph(
 double *area, /* I [sr] Area of source grid */
@@ -417,7 +395,7 @@ nco_grd_lon_typ_enm grd_lon_typ) /* I [num]  */
 
   /* buffers  used in nco-poly_re_org() */
   
-  double pControl[NBR_SPH];
+
 
   poly_sct *pl=(poly_sct*)NULL_CEWI;
 
@@ -512,6 +490,7 @@ nco_grd_lon_typ_enm grd_lon_typ) /* I [num]  */
      * just remember FROM HERE on that the pl->dp_x_ctr, pl->dp_y_ctr iS NOT the Centroid
      * */
 
+    /*
      if(nco_sph_inside_mk(pl,pControl ))
      {
        pl->dp_x_ctr=R2D(pControl[3]);
@@ -519,7 +498,7 @@ nco_grd_lon_typ_enm grd_lon_typ) /* I [num]  */
      }
      else
        nco_poly_ctr_add(pl, grd_lon_typ);
-
+     */
 
 
 
@@ -599,7 +578,7 @@ int *pl_cnt_vrl_ret){
   int max_nbr_vrl=1000;
   int pl_cnt_vrl=0;
 
-  nco_bool bSort=True;
+  //nco_bool bSort=True;
 
   const char fnc_nm[]="nco_poly_mk_vrl()";
 
@@ -637,7 +616,6 @@ int *pl_cnt_vrl_ret){
 
     // cnt_vrl=kd_nearest_intersect(rtree, size, max_nbr_vrl,list,bSort );
 
-
     /* nco_poly_prn(2, pl_lst_in[idx] ); */
 
 
@@ -707,21 +685,22 @@ int *pl_cnt_vrl_ret){
 }
 
 
-poly_sct **
+void **
 nco_poly_lst_mk_vrl_sph(  /* create overlap mesh  for sph polygons */
 poly_sct **pl_lst_in,
 int pl_cnt_in,
 nco_grd_lon_typ_enm grd_lon_typ,
-KDTree *rtree,
+KDTree **tree,
+int nbr_tr,
+int lst_out_typ,
 int *pl_cnt_vrl_ret){
+  /* just duplicate output list to overlap */
+  const char fnc_nm[]="nco_poly_lst_mk_vrl_sph()";
 
-
-/* just duplicate output list to overlap */
   nco_bool bDirtyRats=False;
   nco_bool bSort=True;
 
 
-  int max_nbr_vrl=(NCO_VRL_BLOCKSIZE);
   int pl_cnt_vrl=0;
 
   int thr_idx=0;
@@ -729,15 +708,14 @@ int *pl_cnt_vrl_ret){
   int tot_nan_cnt=0;
   int tot_wrp_cnt=0;
 
+  /* approx number of input cells each thread will process */
+  int thr_quota;
+  /* reporting step */
+  int thr_quota_step;
+
   poly_typ_enm pl_typ;
   size_t idx;
 
-  /* used in realloc */
-  size_t nbr_vrl_blocks=0;
-
-
-  const char fnc_nm[]="nco_poly_lst_mk_vrl_sph()";
-
   int lcl_thr_nbr;
   omp_mem_sct *mem_lst=NULL_CEWI;
 
@@ -746,8 +724,8 @@ int *pl_cnt_vrl_ret){
   double tot_area=0.0;
 
 
-  poly_sct ** pl_lst_vrl=NULL_CEWI;
-  poly_sct **pl_lst_dbg=NULL_CEWI;
+  void** void_lst_vrl=NULL_CEWI;
+  poly_sct** pl_lst_dbg=NULL_CEWI;
 
   FILE * const fp_stderr=stderr;
 
@@ -763,36 +741,40 @@ int *pl_cnt_vrl_ret){
   for(idx=0;idx<lcl_thr_nbr;idx++)
   {
     mem_lst[idx].pl_lst=NULL_CEWI;
+    mem_lst[idx].wgt_lst=NULL_CEWI;
     mem_lst[idx].blk_nbr=0;
     mem_lst[idx].pl_cnt=0;
     mem_lst[idx].kd_list=(KDPriority *)nco_calloc(sizeof(KDPriority),(size_t)(NCO_VRL_BLOCKSIZE));
     mem_lst[idx].kd_cnt=0;
     mem_lst[idx].kd_blk_nbr=1;
-
+    mem_lst[idx].idx_cnt=0;
   }
 
+  thr_quota=pl_cnt_in/lcl_thr_nbr;
+  thr_quota_step=thr_quota/20;
+  if( thr_quota_step <2000 )
+    thr_quota_step=2000;
 
+/* NB: "OpenMP notes" section of nco_rgr.c has detailed discussion of these settings
+     Henry, please keep the variables in alphabetical order within a clause and remember to update Intel */
 #ifdef __GNUG__
   # define GCC_LIB_VERSION ( __GNUC__ * 100 + __GNUC_MINOR__ * 10 + __GNUC_PATCHLEVEL__ )
 # if GCC_LIB_VERSION < 490
 #  define GXX_OLD_OPENMP_SHARED_TREATMENT 1
 # endif /* 480 */
 #endif /* !__GNUC__ */
-#if defined( __INTEL_COMPILER)
-#  pragma omp parallel for default(none) private(idx, thr_idx) shared(bDirtyRats, bSort, fnc_nm ,grd_lon_typ, max_nbr_vrl, mem_lst, pl_cnt_in, pl_lst_in, pl_cnt_dbg, pl_lst_dbg, pl_typ, rtree,  stderr, tot_area, tot_nan_cnt, tot_wrp_cnt)
+#if defined(__INTEL_COMPILER)
+#  pragma omp parallel for private(idx,thr_idx) schedule(dynamic,40) shared(bDirtyRats,bSort,grd_lon_typ,nbr_tr,pl_cnt_dbg,pl_typ,tree,tot_nan_cnt,tot_wrp_cnt)
 #else /* !__INTEL_COMPILER */
 # ifdef GXX_OLD_OPENMP_SHARED_TREATMENT
-#  pragma omp parallel for default(none) private(idx, thr_idx) shared(rtree, grd_lon_typ, bDirtyRats, bSort, max_nbr_vrl, pl_cnt_dbg, tot_nan_cnt, tot_wrp_cnt, pl_typ)
+#  pragma omp parallel for default(none) private(idx,thr_idx) shared(bDirtyRats,bSort,grd_lon_typ,nbr_tr,pl_cnt_dbg,pl_typ,tree,tot_nan_cnt,tot_wrp_cnt)
 # else /* !old g++ */
-#  pragma omp parallel for private(idx, thr_idx) schedule(dynamic,40) shared(rtree, grd_lon_typ, bDirtyRats, bSort, max_nbr_vrl, pl_cnt_dbg, tot_nan_cnt, tot_wrp_cnt, pl_typ)
+#  pragma omp parallel for private(idx,thr_idx) schedule(dynamic,40) shared(bDirtyRats,bSort,grd_lon_typ,nbr_tr,pl_cnt_dbg,pl_typ,tree,tot_nan_cnt,tot_wrp_cnt)
 # endif /* !old g++ */
 #endif /* !__INTEL_COMPILER */
   for(idx=0 ; idx<pl_cnt_in ;idx++ ) {
-
-
     nco_bool bSplit=False;
 
-
     int vrl_cnt = 0;
     int vrl_cnt_on = 0;
     int nan_cnt=0;
@@ -838,9 +820,9 @@ int *pl_cnt_vrl_ret){
 
     /* if a wrapped polygon then do two searches */
     if(bSplit)
-      vrl_cnt = kd_nearest_intersect_wrp(rtree, size1, size2,  &mem_lst[thr_idx]);
+      vrl_cnt = kd_nearest_intersect_wrp(tree, nbr_tr, size1, size2,  &mem_lst[thr_idx]);
     else
-      vrl_cnt = kd_nearest_intersect(rtree, size1, &mem_lst[thr_idx], bSort);
+      vrl_cnt = kd_nearest_intersect(tree, nbr_tr, size1, &mem_lst[thr_idx], bSort);
 
     /* nco_poly_prn(2, pl_lst_in[idx] ); */
 
@@ -852,6 +834,8 @@ int *pl_cnt_vrl_ret){
 
       /* for area debug only */
       mem_lst[thr_idx].kd_list[jdx].area=-1.0;
+      mem_lst[thr_idx].kd_list[jdx].dbg_sng[0]='\0';
+
       /*
       if (pl_lst_in[idx]->pl_typ != pl_out->pl_typ) {
         fprintf(stderr, "%s: %s poly type mismatch\n", nco_prg_nm_get(), fnc_nm);
@@ -891,16 +875,30 @@ int *pl_cnt_vrl_ret){
             pl_vrl = nco_poly_dpl(pl_lst_in[idx]);
 
         }
+
+        if(pl_vrl)
+        {
+          /* add area nb also sets wrapping */
+          nco_poly_minmax_add(pl_vrl, grd_lon_typ, False);
+
+          /* REMEMBER  poly_rll area uses minmax limits AND NOT VERTEX's */
+          nco_poly_area_add(pl_vrl);
+        }
+
+
+
       }
 
       if(pl_typ== poly_sph ) {
 
 
         int lret = 0;
+        int lret2=0;
 
         /* [flg] set by nco_sph_intersect_pre - if True it means that scan hase detected a genuine intersection so
          * so no need to do further processing */
         nco_bool bGenuine=False;
+        nco_bool bBadArea=False;
         char in_sng[VP_MAX];
         char out_sng[VP_MAX];
 
@@ -945,17 +943,37 @@ int *pl_cnt_vrl_ret){
 
         }
 
+        if(pl_vrl)
+        {
+          double min_area= ( pl_out->area < pl_lst_in[idx]->area ? pl_out->area : pl_lst_in[idx]->area  );
+          /* add area nb also sets wrapping */
+          nco_poly_minmax_add(pl_vrl, grd_lon_typ, False);
+
+          /* REMEMBER  poly_rll area uses minmax limits AND NOT VERTEX's */
+          nco_poly_area_add(pl_vrl);
+
+          if( isnan( pl_vrl->area) || pl_vrl->area == 0.0 || (pl_vrl->area - min_area)/ min_area  > 1.0e-04 ) {
+
+            pl_vrl = nco_poly_free(pl_vrl);
+            bBadArea=True;
+          }
+
+        }
+
+
+
         /* swap args around and try again */
-        if(!pl_vrl && bGenuine==False && lret !=1  )
+        if(!pl_vrl &&   ( (bGenuine==False && lret !=1 ) || bBadArea ) )
+          //if(!pl_vrl &&   ( (bGenuine==False && lret !=1 ) || bBadArea ) )
         {
 
           flg_snp_to=1;
 
           nco_sph_intersect_pre(pl_out, pl_lst_in[idx], out_sng);
-          lret = nco_sph_process_pre(pl_lst_in[idx], out_sng, &bGenuine);
+          lret2 = nco_sph_process_pre(pl_lst_in[idx], out_sng, &bGenuine);
 
 
-          switch(lret)
+          switch(lret2)
           {
 
             case 1:
@@ -983,22 +1001,33 @@ int *pl_cnt_vrl_ret){
 
           }
 
-          if(bDirtyRats && pl_vrl && !nco_sph_is_convex(pl_vrl->shp, pl_vrl->crn_nbr) )
+          if(pl_vrl)
           {
-
-            (void) fprintf(stderr, "/************* concave overlap plygon***********/\n");
-            nco_poly_prn(pl_lst_in[idx],0);
-            nco_poly_prn(pl_out,0);
-            nco_poly_prn(pl_vrl, 0);
-            (void) fprintf(stderr, "/***********************************************/\n");
-
+            nco_poly_minmax_add(pl_vrl, grd_lon_typ, False);
+            nco_poly_area_add(pl_vrl);
           }
 
+        }
+
+
+        if(bDirtyRats )
+          sprintf(mem_lst[thr_idx].kd_list[jdx].dbg_sng, "lret=%d in_sng=%s   lret2=%d  out_sng=%s\n",lret, in_sng, lret2, out_sng);
+
+
+        if(bDirtyRats && pl_vrl && !nco_sph_is_convex(pl_vrl->shp, pl_vrl->crn_nbr) )
+        {
+
+          (void) fprintf(stderr, "/************* concave overlap plygon***********/\n");
+          nco_poly_prn(pl_lst_in[idx],0);
+          nco_poly_prn(pl_out,0);
+          nco_poly_prn(pl_vrl, 0);
+          (void) fprintf(stderr, "/***********************************************/\n");
 
         }
 
 
 
+
       } /* end if poly_sph */
 
 
@@ -1014,11 +1043,6 @@ int *pl_cnt_vrl_ret){
 
 
 
-        /* add area nb also sets wrapping */
-        nco_poly_minmax_add(pl_vrl, grd_lon_typ, False);
-
-        /* REMEMBER  poly_rll area uses minmax limits AND NOT VERTEX's */
-        nco_poly_area_add(pl_vrl);
 
         /* shp not needed */
         nco_poly_shp_free(pl_vrl);
@@ -1037,7 +1061,7 @@ int *pl_cnt_vrl_ret){
           pl_vrl->area=0.0;
 
           pl_vrl=nco_poly_free(pl_vrl);
-           continue;
+          continue;
         }
 
         /* for input polygon wgt is used to calculate frac_a */
@@ -1045,30 +1069,45 @@ int *pl_cnt_vrl_ret){
 
 
 
+        /*
 #ifdef _OPENMP
 #pragma omp critical
 #endif
         {
-          /* This is critcal as two threads can add to pl_out->wgt possibly at the same time
-           * we wish to avoid a collision */
-          /* for output  polygon wgt is used to calculate frac_b *
-           *
-           * maybe we can move this code back up to nco_msh_mk() ? */
           pl_out->wgt+=pl_vrl->wgt;
 
         }
+*/
 
         vrl_area += pl_vrl->area;
+        mem_lst[thr_idx].kd_list[jdx].area=pl_vrl->area;
 
-        if( mem_lst[thr_idx].blk_nbr * NCO_VRL_BLOCKSIZE <  mem_lst[thr_idx].pl_cnt +1 )
-          mem_lst[thr_idx].pl_lst= (poly_sct**)nco_realloc(mem_lst[thr_idx].pl_lst, sizeof(poly_sct*) * ++mem_lst[thr_idx].blk_nbr * NCO_VRL_BLOCKSIZE );
 
-        mem_lst[thr_idx].pl_lst[  mem_lst[thr_idx].pl_cnt  ]=pl_vrl;
-        mem_lst[thr_idx].pl_cnt++;
+        if( mem_lst[thr_idx].blk_nbr * NCO_VRL_BLOCKSIZE <  mem_lst[thr_idx].pl_cnt +1 ){
+          if(lst_out_typ==1)
+            mem_lst[thr_idx].wgt_lst= (wgt_sct**)nco_realloc(mem_lst[thr_idx].wgt_lst, sizeof(wgt_sct*) * ++mem_lst[thr_idx].blk_nbr * NCO_VRL_BLOCKSIZE );
+          else if(lst_out_typ==2)
+            mem_lst[thr_idx].pl_lst= (poly_sct**)nco_realloc(mem_lst[thr_idx].pl_lst, sizeof(poly_sct*) * ++mem_lst[thr_idx].blk_nbr * NCO_VRL_BLOCKSIZE );
+        }
+
+
+        if(lst_out_typ==1) {
+          wgt_sct *wgt_lcl=(wgt_sct*)nco_calloc(1,sizeof(wgt_sct));
+          wgt_lcl->src_id=pl_vrl->src_id;
+          wgt_lcl->dst_id=pl_vrl->dst_id;
+          wgt_lcl->area=pl_vrl->area;
+          wgt_lcl->wgt=pl_vrl->wgt;
+          mem_lst[thr_idx].wgt_lst[mem_lst[thr_idx].pl_cnt++] =wgt_lcl;
+
+          pl_vrl=nco_poly_free(pl_vrl);
+        }
+        else if(lst_out_typ==2 )
+          mem_lst[thr_idx].pl_lst[mem_lst[thr_idx].pl_cnt++] = pl_vrl;
+
         vrl_cnt_on++;
 
         /* for area debug only */
-        mem_lst[thr_idx].kd_list[jdx].area=pl_vrl->area;
+
 
 
 
@@ -1093,6 +1132,7 @@ int *pl_cnt_vrl_ret){
       } /* end OMP critical */
 
       /* area diff by more than 10% */
+      int kdx;
       double eps = 1e-8;
       double frc = vrl_area / pl_lst_in[idx]->area;
       if (frc < (1 - eps) || frc > 1 + eps) {
@@ -1104,32 +1144,50 @@ int *pl_cnt_vrl_ret){
 
         if (bDirtyRats) {
           //if (pl_lst_in[idx]->bwrp ) {
-          pl_lst_dbg = (poly_sct **) nco_realloc(pl_lst_dbg, sizeof(poly_sct *) * (pl_cnt_dbg + 1));
-          pl_lst_dbg[pl_cnt_dbg] = nco_poly_dpl(pl_lst_in[idx]);
-          pl_cnt_dbg++;
 
           if (1) {
-            int kdx;
-            (void) fprintf(stderr, "# /** following pl_lst_in[%lu]  **/\n", idx);
+            (void) fprintf(fp_stderr, "# /** following pl_lst_in[%lu]  **/\n", idx);
             nco_poly_prn(pl_lst_in[idx], 0);
-            (void) fprintf(stderr, "# /** potential overlaps to  follow  **/\n");
+            (void) fprintf(fp_stderr, "# /** overlaps to  follow  **/\n");
             for (kdx = 0; kdx < vrl_cnt; kdx++) {
               nco_poly_prn((poly_sct *) mem_lst[thr_idx].kd_list[kdx].elem->item, 0);
               (void)fprintf(fp_stderr, "# vrl_area=%.15e\n",mem_lst[thr_idx].kd_list[kdx].area );
+              (void)fprintf(fp_stderr, "# dbg_sng=%s\n",mem_lst[thr_idx].kd_list[kdx].dbg_sng );
+
             }
 
             (void) fprintf(stderr, "/************* end dirty rats ***************/\n");
           }
 
+          if(1 && vrl_cnt_on>0 && lst_out_typ==2 ) {
+
+
+            pl_lst_dbg = (poly_sct **) nco_realloc(pl_lst_dbg, sizeof(poly_sct *) * (pl_cnt_dbg + vrl_cnt_on));
+
+            /* write overlaps maybe */
+            for (kdx = 0; kdx < vrl_cnt_on; kdx++) {
+              poly_sct * lcl_poly=mem_lst[thr_idx].pl_lst[mem_lst[thr_idx].pl_cnt - vrl_cnt_on + kdx];
+
+              if(lcl_poly->src_id== pl_lst_in[idx]->src_id )
+                pl_lst_dbg[pl_cnt_dbg++] = nco_poly_dpl(lcl_poly);
+
+            }
+
+
+
+          }
+
+
 
         }
 
       }
 
-
     } /* end dbg */
 
-
+    /* output some usefull tracking stuff - not debug but informative */
+    if (  ++mem_lst[thr_idx].idx_cnt % thr_quota_step == 0 && nco_dbg_lvl_get() >=3   )
+      (void)fprintf(fp_stderr, "%s: thread %d  has processed %2.2f%% (%ld) of src cells quota and output %ld overlap cells\n",    nco_prg_nm_get(), thr_idx, (float)mem_lst[thr_idx].idx_cnt/(float)thr_quota *100.0,  mem_lst[thr_idx].idx_cnt, mem_lst[thr_idx].pl_cnt  );
 
   } /* end for idx */
 
@@ -1140,71 +1198,34 @@ int *pl_cnt_vrl_ret){
   if (nco_dbg_lvl_get() >= nco_dbg_dev)
     (void) fprintf(stderr, "%s: total overlaps=%d, total_area=%.15f (area=%3.10f%%) total num wrapped= %d total nan nbr=%d \n", nco_prg_nm_get(), pl_cnt_vrl, tot_area, tot_area /4.0 / M_PI *100.0, tot_wrp_cnt, tot_nan_cnt);
 
-
-
-
   /* write filtered polygons to file */
   if(bDirtyRats && pl_cnt_dbg)
   {
-
-
-    nco_msh_poly_lst_wrt("tst-wrt-dbg.nc", pl_lst_dbg, pl_cnt_dbg, grd_lon_typ);
+    nco_msh_poly_lst_wrt("tst-wrt-dbg.nc", pl_lst_dbg, pl_cnt_dbg, grd_lon_typ,NC_FORMAT_NETCDF4);
     pl_lst_dbg=(poly_sct**)nco_poly_lst_free(pl_lst_dbg, pl_cnt_dbg);
   }
 
-
-
-
-  /* concatenate memory lists */
-  {
-
-    size_t tot_pl_cnt = 0;
-    poly_sct **tmp_pl_lst = NULL_CEWI;
-
-    /* find total size of lists */
-    for (idx = 0; idx < lcl_thr_nbr; idx++)
-      tot_pl_cnt += mem_lst[idx].pl_cnt;
-
-
-    pl_lst_vrl = mem_lst[0].pl_lst;
-
-
-    pl_lst_vrl = (poly_sct **) nco_realloc(pl_lst_vrl, sizeof(poly_sct *) * tot_pl_cnt);
-
-
-    tmp_pl_lst = pl_lst_vrl;
-    tmp_pl_lst += mem_lst[0].pl_cnt;
-
-    for (idx = 1; idx < lcl_thr_nbr; idx++) {
-      if (mem_lst[idx].pl_lst) {
-        memcpy(tmp_pl_lst, mem_lst[idx].pl_lst, sizeof(poly_sct *) * mem_lst[idx].pl_cnt);
-        tmp_pl_lst += mem_lst[idx].pl_cnt;
-
-        /* free up list */
-        mem_lst[idx].pl_lst = (poly_sct **) nco_free(mem_lst[idx].pl_lst);
-      }
-    }
-
-    *pl_cnt_vrl_ret=tot_pl_cnt;
-  }
-
-
-
+  /* concatenate memory list, place results into first member list */
+  nco_mem_lst_cat(mem_lst, lcl_thr_nbr);
 
   /* free up kd_list's */
   for(idx=0;idx<lcl_thr_nbr;idx++)
     mem_lst[idx].kd_list= (KDPriority*) nco_free(mem_lst[idx].kd_list);
 
+  *pl_cnt_vrl_ret=mem_lst[0].pl_cnt;
+
+  if(lst_out_typ==1)
+    void_lst_vrl=(void**) mem_lst[0].wgt_lst;
+  else if( lst_out_typ==2 )
+    void_lst_vrl=(void**) mem_lst[0].pl_lst;
 
   mem_lst=(omp_mem_sct*)nco_free(mem_lst);
 
-  return pl_lst_vrl;
-
+  /* REMEMBER the void type can be a wgt_sct** array or a poly_sct** array
+   * wgt_sct is a subset of poly_sct - with simple members */
+  return void_lst_vrl;
 }
 
-
-
-
 /* check areas - nb WARNING modifies area in pl_lst_in and pl_lst_out */
 void nco_poly_lst_chk(
 poly_sct **pl_lst_in,
@@ -1218,7 +1239,6 @@ int pl_cnt_vrl)
   int idx;
   int jdx;
 
-  double sum=0.0;
   double epsilon=1.0e-8;
 
   const char fnc_nm[]="nco_poly_lst_chk()";
@@ -1258,10 +1278,6 @@ int pl_cnt_vrl)
     if( fabs(  pl_lst_out[idx]->area) > epsilon)
       fprintf(stderr, "src_id=%d area=%.10f\n", pl_lst_out[idx]->src_id, pl_lst_out[idx]->area );
 
-
-
-
-
    return;
 }
 
@@ -1334,6 +1350,346 @@ int *pl_cnt_dbg) /* size of output dbg grid */
   *pl_cnt_dbg=pl_nbr_dbg;
 
   return pl_lst_dbg;
-
-
 }
+
+
+wgt_sct **
+nco_poly_lst_mk_dwe_sph(
+rgr_sct *const rgr_nfo,
+poly_sct **pl_lst_out,
+int pl_cnt,
+nco_grd_lon_typ_enm grd_lon_typ,
+KDTree **tree,
+int nbr_tr,
+int *wgt_cnt_bln_ret) {
+
+  /* just duplicate output list to overlap */
+  const char fnc_nm[] = "nco_poly_lst_mk_dwe_sph()";
+
+  int thr_idx = 0;
+  /* approx number of input cells each thread will process */
+  int thr_quota;
+  /* reporting step */
+  int thr_quota_step;
+  /* max number of nearest neighbours to consider - nr-reference from rgr_nfo */
+  int const max_nbr_dwe=20;
+  int nbr_dwe=0;
+
+  double pow_dwe=0.0;
+
+  double min_dist=1.0e-12;
+  double min_wgt=1.0e-20;
+
+  poly_typ_enm pl_typ;
+  size_t idx;
+
+  int lcl_thr_nbr;
+  omp_mem_sct *mem_lst = NULL_CEWI;
+
+  wgt_sct **wgt_lst_dwe = NULL_CEWI;
+
+  FILE *const fp_stderr = stderr;
+
+  pl_typ = pl_lst_out[0]->pl_typ;
+
+  lcl_thr_nbr = omp_get_max_threads();
+
+  nbr_dwe= ( rgr_nfo->xtr_nsp > max_nbr_dwe ? max_nbr_dwe : rgr_nfo->xtr_nsp );
+
+  pow_dwe=rgr_nfo->xtr_xpn;
+
+  mem_lst = (omp_mem_sct *) nco_malloc(sizeof(omp_mem_sct) * lcl_thr_nbr);
+
+  for (idx = 0; idx < lcl_thr_nbr; idx++) {
+    mem_lst[idx].wgt_lst = NULL_CEWI;
+    mem_lst[idx].blk_nbr = 0;
+    mem_lst[idx].pl_cnt = 0;
+    mem_lst[idx].kd_list = (KDPriority *) nco_calloc(sizeof(KDPriority), (size_t) (NCO_VRL_BLOCKSIZE));
+    mem_lst[idx].kd_cnt = 0;
+    mem_lst[idx].kd_blk_nbr = 1;
+    mem_lst[idx].idx_cnt = 0;
+  }
+
+  thr_quota = pl_cnt / lcl_thr_nbr;
+  thr_quota_step = thr_quota / 20;
+  if (thr_quota_step < 2000)
+    thr_quota_step = 2000;
+  
+  /* NB: "OpenMP notes" section of nco_rgr.c has detailed discussion of these settings
+     Henry, please keep the variables in alphabetical order within a clause and remember to update Intel */
+#ifdef __GNUG__
+  # define GCC_LIB_VERSION ( __GNUC__ * 100 + __GNUC_MINOR__ * 10 + __GNUC_PATCHLEVEL__ )
+# if GCC_LIB_VERSION < 490
+#  define GXX_OLD_OPENMP_SHARED_TREATMENT 1
+# endif /* 480 */
+#endif /* !__GNUC__ */
+#if defined(__INTEL_COMPILER)
+#  pragma omp parallel for private(idx,thr_idx) schedule(dynamic,40) shared(grd_lon_typ,nbr_tr,pl_typ,tree)
+#else /* !__INTEL_COMPILER */
+# ifdef GXX_OLD_OPENMP_SHARED_TREATMENT
+#  pragma omp parallel for default(none) private(idx,thr_idx) shared(bDirtyRats,grd_lon_typ,max_nbr_vrl,nbr_tr,pl_cnt_dbg,pl_typ,tree,tot_nan_cnt,tot_wrp_cnt)
+# else /* !old g++ */
+#  pragma omp parallel for private(idx,thr_idx) schedule(dynamic,40) shared(grd_lon_typ,nbr_tr,pl_typ,tree)
+# endif /* !old g++ */
+#endif /* !__INTEL_COMPILER */
+  for (idx = 0; idx < pl_cnt; idx++) {
+
+    double dp_x_wrp;   /* used to do a wrapped lon search */
+    double wgt_ttl=0.0;
+
+    int nbr_dwe_cnt; /* equal to or less than nbr_nni */
+
+    wgt_sct wgt_pre[max_nbr_dwe];
+
+    wgt_sct * wgt_lcl=NULL_CEWI;
+    poly_sct *pl=NULL_CEWI;
+
+    size_t jdx;
+    size_t kdx;
+    size_t nbr_lst_lcl;
+
+    // (void) nco_poly_set_priority(max_nbr_vrl, list);
+
+    thr_idx = omp_get_thread_num();
+
+    if (0 && nco_dbg_lvl_get() >= nco_dbg_dev)
+      fprintf(fp_stderr, "%s(): idx=%lu thr=%d\n", fnc_nm, idx, thr_idx);
+
+    if (pl_lst_out[idx]->bmsk == False)
+      continue;
+
+    mem_lst[thr_idx].kd_cnt = 0;
+
+    if (mem_lst[thr_idx].kd_blk_nbr > 1) {
+      mem_lst[thr_idx].kd_blk_nbr = 1;
+
+      mem_lst[thr_idx].kd_list = (KDPriority *) nco_free(mem_lst[thr_idx].kd_list);
+      //mem_lst[thr_idx].kd_list=(KDPriority*)nco_realloc(mem_lst[idx].kd_list, sizeof(KDPriority) * NCO_VRL_BLOCKSIZE );
+      mem_lst[idx].kd_list = (KDPriority *) nco_calloc(sizeof(KDPriority), (size_t) (NCO_VRL_BLOCKSIZE));
+    }
+
+    /* get bounds of polygon in */
+    //bSplit = nco_poly_minmax_split(pl_lst_out[idx], grd_lon_typ, size1, size2);
+
+    dp_x_wrp=KD_DBL_MAX;
+
+    for(kdx=0;kdx<nbr_tr;kdx++)
+      kd_nearest(tree[kdx], pl_lst_out[idx]->dp_x_ctr, pl_lst_out[idx]->dp_y_ctr, pl_typ,  nbr_dwe, mem_lst[thr_idx].kd_list + nbr_dwe *kdx );
+
+    nbr_lst_lcl=nbr_dwe*nbr_tr;
+
+    switch(grd_lon_typ)
+    {
+      case nco_grd_lon_nil:
+      case nco_grd_lon_unk:
+      case nco_grd_lon_Grn_ctr:
+      case nco_grd_lon_Grn_wst:
+      case nco_grd_lon_bb:
+        if(pl_lst_out[idx]->dp_x_ctr <180.0)
+          dp_x_wrp=pl_lst_out[idx]->dp_x_ctr+360.0;
+        else if(pl_lst_out[idx]->dp_x_ctr >180.0)
+          dp_x_wrp=pl_lst_out[idx]->dp_x_ctr-360.0;
+        break;
+
+      case nco_grd_lon_180_wst:
+      case nco_grd_lon_180_ctr:
+        if(pl_lst_out[idx]->dp_x_ctr <0.0)
+          dp_x_wrp=pl_lst_out[idx]->dp_x_ctr+360.0;
+        else if(pl_lst_out[idx]->dp_x_ctr >0.0)
+          dp_x_wrp=pl_lst_out[idx]->dp_x_ctr-360.0;
+        break;
+    }
+
+    if(dp_x_wrp != KD_DBL_MAX)
+    {
+      for (kdx = 0; kdx < nbr_tr; kdx++)
+        kd_nearest(tree[kdx], dp_x_wrp, pl_lst_out[idx]->dp_y_ctr, pl_typ, nbr_dwe, mem_lst[thr_idx].kd_list + nbr_lst_lcl+nbr_dwe * kdx);
+
+      nbr_lst_lcl+=nbr_dwe*nbr_tr;
+    }
+
+    if(nbr_tr >1 )
+      qsort((void *)mem_lst[thr_idx].kd_list, nbr_lst_lcl,  sizeof(KDPriority), kd_priority_cmp_dist);
+
+    /* remember kd list sorted according to distance */
+
+    /* check first member distance if min then output singleton */
+    if(mem_lst[thr_idx].kd_list[0].dist <= min_dist)
+    {
+
+      pl=(poly_sct*)mem_lst[thr_idx].kd_list[0].elem->item;
+
+      wgt_lcl=(wgt_sct*)nco_malloc(sizeof(wgt_sct));
+      wgt_lcl->src_id=pl->src_id;
+      wgt_lcl->dst_id=pl_lst_out[idx]->src_id;
+      wgt_lcl->area=pl->area;
+      wgt_lcl->dist=mem_lst[thr_idx].kd_list[0].dist;
+      wgt_lcl->wgt=1.0;
+
+      if( mem_lst[thr_idx].blk_nbr * NCO_VRL_BLOCKSIZE <  mem_lst[thr_idx].pl_cnt +1 )
+        mem_lst[thr_idx].wgt_lst= (wgt_sct**)nco_realloc(mem_lst[thr_idx].wgt_lst, sizeof(wgt_sct*) * ++mem_lst[thr_idx].blk_nbr * NCO_VRL_BLOCKSIZE );
+
+      mem_lst[thr_idx].wgt_lst[mem_lst[thr_idx].pl_cnt++] =wgt_lcl;
+
+      if (nco_dbg_lvl_get() >= nco_dbg_dev)
+        (void)fprintf(fp_stderr,"%s:%s: singleton  x_ctr=%f  y_ctr=%f\n", nco_prg_nm_get(), fnc_nm, pl_lst_out[idx]->dp_x_ctr, pl_lst_out[idx]->dp_y_ctr );
+
+    }else{
+
+      /* check for duplicates in first nbr_nni by sorting again with ->item  !!!*/
+      kd_priority_list_sort(mem_lst[thr_idx].kd_list,nbr_dwe, nbr_dwe,&nbr_dwe_cnt );
+
+      if (nco_dbg_lvl_get() >= nco_dbg_dev && nbr_dwe_cnt < nbr_dwe )
+         (void)fprintf(fp_stderr,"%s:%s: nbr_nni_cnt=%d x_ctr=%f  y_ctr=%f\n", nco_prg_nm_get(), fnc_nm, nbr_dwe_cnt, pl_lst_out[idx]->dp_x_ctr, pl_lst_out[idx]->dp_y_ctr );
+
+      /* output at least one */
+      for (jdx = 0; jdx < nbr_dwe_cnt; jdx++) {
+
+        pl = (poly_sct *) mem_lst[thr_idx].kd_list[jdx].elem->item;
+
+        /* remember src is in tree and dst is in list */
+        wgt_pre[jdx].src_id = pl->src_id;
+        wgt_pre[jdx].dst_id = pl_lst_out[idx]->src_id;
+        wgt_pre[jdx].area = pl->area;
+        wgt_pre[jdx].dist = mem_lst[thr_idx].kd_list[jdx].dist;
+        /* use dist squared */
+        wgt_pre[jdx].wgt = 1.0 /  pow(wgt_pre[jdx].dist, pow_dwe);
+      }
+
+      /* find weights total */
+      for (jdx = 0; jdx < nbr_dwe_cnt; jdx++)
+        wgt_ttl += wgt_pre[jdx].wgt;
+
+      /* normalize weights */
+      for (jdx = 0; jdx < nbr_dwe_cnt; jdx++)
+        wgt_pre[jdx].wgt /= wgt_ttl;
+
+      for (jdx = 0; jdx < nbr_dwe_cnt; jdx++) {
+
+        if (wgt_pre[jdx].wgt < min_wgt)
+          continue;
+
+        wgt_lcl = (wgt_sct *) nco_malloc(sizeof(wgt_sct));
+        *wgt_lcl = wgt_pre[jdx];
+
+        if (mem_lst[thr_idx].blk_nbr * NCO_VRL_BLOCKSIZE < mem_lst[thr_idx].pl_cnt + 1)
+          mem_lst[thr_idx].wgt_lst = (wgt_sct **) nco_realloc(mem_lst[thr_idx].wgt_lst,sizeof(wgt_sct *) * ++mem_lst[thr_idx].blk_nbr *NCO_VRL_BLOCKSIZE);
+
+        mem_lst[thr_idx].wgt_lst[mem_lst[thr_idx].pl_cnt++] = wgt_lcl;
+
+        //(void)fprintf(stderr,"%s: weight(%lu)=%f ",fnc_nm, mem_lst[thr_idx].pl_cnt, wgt_lcl->wgt);
+
+      } /* end jdx */
+    }
+
+    /* output some usefull tracking stuff - not debug but informative */
+    if (  ++mem_lst[thr_idx].idx_cnt % thr_quota_step == 0 && nco_dbg_lvl_get() >=3   )
+      (void)fprintf(fp_stderr, "%s: thread %d  has processed %2.2f%% (%ld) of src cells quota and output %ld overlap cells\n",    nco_prg_nm_get(), thr_idx, (float)mem_lst[thr_idx].idx_cnt/(float)thr_quota *100.0,  mem_lst[thr_idx].idx_cnt, mem_lst[thr_idx].pl_cnt  );
+  }  /* end idx */
+
+  nco_mem_lst_cat(mem_lst, lcl_thr_nbr);
+
+  /* free up kd_list's */
+  for(idx=0;idx<lcl_thr_nbr;idx++)
+    mem_lst[idx].kd_list= (KDPriority*) nco_free(mem_lst[idx].kd_list);
+
+  wgt_lst_dwe=mem_lst[0].wgt_lst;
+
+  *wgt_cnt_bln_ret=mem_lst[0].pl_cnt;
+
+  mem_lst=(omp_mem_sct*)nco_free(mem_lst);
+
+  return wgt_lst_dwe;
+} /* !nco_poly_lst_mk_dwe_sph() */
+
+void nco_poly_lst_ctr_add(
+poly_sct **pl_lst,
+int pl_cnt,
+int ctr_typ)
+{
+  int idx;
+
+  double pControl[NBR_SPH];
+
+  for(idx=0;idx<pl_cnt;idx++)
+  {
+    if(pl_lst[idx]->crn_nbr <3 || pl_lst[idx]->area==0.0  )
+      continue;
+
+    if(ctr_typ==1){
+      nco_sph_inside_mk(pl_lst[idx], pControl);
+      pl_lst[idx]->dp_x_ctr=R2D(pControl[3]);
+      pl_lst[idx]->dp_y_ctr=R2D(pControl[4]);
+    }
+  }
+  return;
+} /* !nco_poly_lst_ctr_add() */
+
+void
+nco_mem_lst_cat(
+omp_mem_sct *mem_lst,
+int sz_lst)
+{
+  int idx;
+  int i_typ;
+  size_t tot_cnt = 0;
+
+  if(mem_lst->wgt_lst)
+    i_typ = 1;
+  else if(mem_lst->pl_lst)
+    i_typ = 2;
+  else
+    i_typ=0;
+
+  /* quick return if list empty */
+  if(!i_typ)
+    return;
+
+  /* find total size of lists */
+  for (idx = 0; idx < sz_lst; idx++)
+    tot_cnt += mem_lst[idx].pl_cnt;
+
+  if(!tot_cnt)
+    return;
+
+  else if( i_typ==1 ){
+
+    wgt_sct **tmp_wgt_lst = NULL;
+
+    tmp_wgt_lst = mem_lst[0].wgt_lst = (wgt_sct **) nco_realloc(mem_lst[0].wgt_lst, sizeof(wgt_sct *) * tot_cnt);
+
+    tmp_wgt_lst += mem_lst[0].pl_cnt;
+
+    for (idx = 1; idx < sz_lst; idx++) {
+      if (mem_lst[idx].wgt_lst) {
+        memcpy(tmp_wgt_lst, mem_lst[idx].wgt_lst, sizeof(wgt_sct *) * mem_lst[idx].pl_cnt);
+        tmp_wgt_lst += mem_lst[idx].pl_cnt;
+        /* free up list */
+        mem_lst[idx].wgt_lst = (wgt_sct **) nco_free(mem_lst[idx].wgt_lst);
+      }
+    }
+  }
+
+  else if(i_typ==2) {
+    poly_sct **tmp_ply_lst = NULL;
+
+    tmp_ply_lst = mem_lst[0].pl_lst = (poly_sct **) nco_realloc(mem_lst[0].pl_lst, sizeof(poly_sct *) * tot_cnt);
+
+    tmp_ply_lst += mem_lst[0].pl_cnt;
+
+    for (idx = 1; idx < sz_lst; idx++) {
+      if (mem_lst[idx].pl_lst) {
+        memcpy(tmp_ply_lst, mem_lst[idx].pl_lst, sizeof(poly_sct *) * mem_lst[idx].pl_cnt);
+        tmp_ply_lst += mem_lst[idx].pl_cnt;
+        /* free up list */
+        mem_lst[idx].pl_lst = (poly_sct **) nco_free(mem_lst[idx].pl_lst);
+      }
+    }
+  }
+
+  /* update first list with total */
+  mem_lst[0].pl_cnt=tot_cnt;
+
+  return;
+} /* !nco_mem_lst_cat() */
+
