@@ -623,9 +623,11 @@ nco_ppc_bitmask /* [fnc] Mask-out insignificant bits of significand */
   unsigned int *u32_ptr;
   unsigned int msk_f32_u32_zro;
   unsigned int msk_f32_u32_one;
+  unsigned int msk_f32_u32_hshv;
   unsigned long int *u64_ptr;
   unsigned long int msk_f64_u64_zro;
   unsigned long int msk_f64_u64_one;
+  unsigned long int msk_f64_u64_hshv;
   unsigned short prc_bnr_ceil; /* [nbr] Exact binary digits of precision rounded-up */
   unsigned short prc_bnr_xpl_rqr; /* [nbr] Explicitly represented binary digits required to retain */
   
@@ -673,6 +675,7 @@ nco_ppc_bitmask /* [fnc] Mask-out insignificant bits of significand */
     msk_f32_u32_zro <<= bit_xpl_nbr_zro;
     /* Bit Set   mask for OR:  Put ones into bits to be set, zeros in untouched bits */
     msk_f32_u32_one=~msk_f32_u32_zro;
+    msk_f32_u32_hshv=msk_f32_u32_one & (msk_f32_u32_zro >> 1); /* Set one bit: the MSB of LSBs */
     if(nco_baa_cnv_get() == nco_baa_grm){
       /* Bit-Groom: alternately shave and set LSBs */
       if(!has_mss_val){
@@ -756,6 +759,7 @@ nco_ppc_bitmask /* [fnc] Mask-out insignificant bits of significand */
 	  if(nco_dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: DEBUG nco_ppc_bitmask() reports val = %g, u32_xpn = %u, i32_xpn=%d, dgt_nbr_pre_dcm = d_i = %d, qnt_pwr_xpn = p_i = %d, qnt_fct_flt = q_i = %g\n",nco_prg_nm_get(),op1.fp[idx],u32_xpn,i32_xpn,dgt_nbr_pre_dcm,qnt_pwr_xpn,qnt_fct_flt);
 	} /* !idx */
 	for(idx=1L;idx<sz;idx+=2L){
+	  u8_xpn='0'; /* CEWI */
 	  if(nco_dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: DEBUG nco_ppc_bitmask() reports val = %g, u8_xpn_nbr = %hhu\n",nco_prg_nm_get(),op1.fp[idx],u8_xpn);
 	  if(u32_ptr[idx] != 0U) /* Never quantize upwards floating point values of zero */
 	    u32_ptr[idx]|=msk_f32_u32_one;
@@ -773,9 +777,45 @@ nco_ppc_bitmask /* [fnc] Mask-out insignificant bits of significand */
 	 ccc --tst=bnr --flt_foo=8 2> /dev/null | grep "Binary of float"
 	 ncks -O -C -D 1 --baa=4 -v ppc_bgr --ppc default=3 ~/nco/data/in.nc ~/foo.nc
 	 ncks -O -C -D 1 --baa=4 -v one_dmn_rec_var_flt --ppc default=3 ~/nco/data/in.nc ~/foo.nc */
+      idx=0L;
       if(nco_dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: DEBUG nco_ppc_bitmask() reports val = %g\n",nco_prg_nm_get(),op1.fp[idx]);
+    }else if(nco_baa_cnv_get() == nco_baa_rnd){
+      /* Round mantissa, LSBs to zero contributed by Rostislav Kouznetsov 20200711
+	 Round mantissa using floating-point arithmetic, shave LSB using bit-mask */
+      if(!has_mss_val){
+	for(idx=0L;idx<sz;idx++){
+          u32_ptr[idx]+=msk_f32_u32_hshv; /* Add 1 at MSB of LSBs, carry 1 to mantissa or even exponent*/
+          u32_ptr[idx]&=msk_f32_u32_zro; /* Shave it */
+        } /* !idx */
+      }else{ /* !has_mss_val */
+	const float mss_val_flt=*mss_val.fp;
+	for(idx=0L;idx<sz;idx++){
+	  if(op1.fp[idx] != mss_val_flt){ 
+            u32_ptr[idx]+=msk_f32_u32_hshv;/* add 1 to the MSB of LSBs*/
+            u32_ptr[idx]&=msk_f32_u32_zro; /* Shave it */
+	  } /* !mss_val_flt */
+	} /* !idx */
+      } /* !has_mss_val */
+    }else if(nco_baa_cnv_get() == nco_baa_sh2){
+      /* Bit Half-Shave contributed by Rostislav Kouznetsov 20200715
+	 Shave LSBs and set MSB of them
+	 See figures at https://github.com/nco/nco/pull/200 */
+      if(!has_mss_val){
+	for(idx=0L;idx<sz;idx++){
+	  u32_ptr[idx]&=msk_f32_u32_zro; /* Shave as normal */
+	  u32_ptr[idx]|=msk_f32_u32_hshv; /* Set MSB of LSBs */
+        } /* !idx */
+      }else{
+	const float mss_val_flt=*mss_val.fp;
+	for(idx=0L;idx<sz;idx++){
+	  if(op1.fp[idx] != mss_val_flt){
+	    u32_ptr[idx]&=msk_f32_u32_zro; /* Shave as normal */
+	    u32_ptr[idx]|=msk_f32_u32_hshv; /* Set MSB of LSBs */
+          } /* !mss_val_flt */
+	} /* !idx */
+      } /* !has_mss_val */
     }else abort();
-    break;
+    break; /* !NC_FLOAT */
   case NC_DOUBLE:
     bit_xpl_nbr_sgn=bit_xpl_nbr_sgn_dbl;
     bit_xpl_nbr_zro=bit_xpl_nbr_sgn-prc_bnr_xpl_rqr;
@@ -788,6 +828,7 @@ nco_ppc_bitmask /* [fnc] Mask-out insignificant bits of significand */
     msk_f64_u64_zro <<= bit_xpl_nbr_zro;
     /* Bit Set   mask for OR:  Put ones into bits to be set, zeros in untouched bits */
     msk_f64_u64_one=~msk_f64_u64_zro;
+    msk_f64_u64_hshv=msk_f64_u64_one & (msk_f64_u64_zro >> 1); /* Set one bit: the MSB of LSBs */
     if(nco_baa_cnv_get() == nco_baa_grm){
       /* Bit-Groom: alternately shave and set LSBs */
       if(!has_mss_val){
@@ -836,8 +877,44 @@ nco_ppc_bitmask /* [fnc] Mask-out insignificant bits of significand */
 	for(idx=1L;idx<sz;idx+=2L)
 	  if(op1.dp[idx] != mss_val_dbl && u64_ptr[idx] != 0UL) u64_ptr[idx]|=msk_f64_u64_one;
       } /* end else */
+    }else if(nco_baa_cnv_get() == nco_baa_rnd){
+      /* Round mantissa, LSBs to zero contributed by Rostislav Kouznetsov 20200711
+	 Round mantissa using floating-point arithmetic, shave LSB using bit-mask
+	 See figures at https://github.com/nco/nco/pull/199 */
+      if(!has_mss_val){
+	for(idx=0L;idx<sz;idx++){
+          u64_ptr[idx]+=msk_f64_u64_hshv; /* Add 1 at MSB of LSBs */
+          u64_ptr[idx]&=msk_f64_u64_zro; /* Shave it */
+        } /* !idx */
+      }else{ /* !has_mss_val */
+	const double mss_val_dbl=*mss_val.dp;
+	for(idx=0L;idx<sz;idx++){
+	  if(op1.dp[idx] != mss_val_dbl){ 
+            u64_ptr[idx]+=msk_f64_u64_hshv; /* Add 1 at MSB of LSBs */
+            u64_ptr[idx]&=msk_f64_u64_zro; /* Shave it */
+	  } /* !mss_val_dbl */
+	} /* !idx */
+      } /* !has_mss_val */
+    }else if(nco_baa_cnv_get() == nco_baa_sh2){
+      /* Bit Half-Shave contributed by Rostislav Kouznetsov 20200715
+	 Shave LSBs and set MSB of them
+	 See figures at https://github.com/nco/nco/pull/200 */
+      if(!has_mss_val){
+	for(idx=0L;idx<sz;idx++){
+	  u64_ptr[idx]&=msk_f64_u64_zro; /* Shave as normal */
+	  u64_ptr[idx]|=msk_f64_u64_hshv; /* Set MSB of LSBs */
+        } /* !idx */
+      }else{
+	const double mss_val_dbl=*mss_val.dp;
+	for(idx=0L;idx<sz;idx++){
+	  if(op1.dp[idx] != mss_val_dbl){
+	    u64_ptr[idx]&=msk_f64_u64_zro; /* Shave as normal */
+	    u64_ptr[idx]|=msk_f64_u64_hshv; /* Set MSB of LSBs */
+          } /* !mss_val_dbl */
+	} /* !idx */
+      } /* !has_mss_val */
     }else abort();
-    break;
+    break; /* !NC_DOUBLE */
   case NC_INT: /* Do nothing for non-floating point types ...*/
   case NC_SHORT:
   case NC_CHAR:
